#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from naruno.config import *
from naruno.lib.config_system import get_config

temp_json = {
    "test_mode": False,
    "funtionaltest_mode": False,
    "debug_mode": False,
    "wallet": 0,
    "save_blockshash": True,
    "mute_notifications": False,
    "dark_mode": True,
    "status_cache_time": 0,
    "status_cache": {},
    "status_working": False,
    "publisher_mode": False,
    "baklava": False,
    "disable_log_clearing": False,
    "baklava_users": False,
    "dont_save_blocks": False,
}


def save_settings(new_settings):
    """
    Saves the settings.
    """

    os.chdir(get_config()["main_folder"])
    with open(SETTING_PATH, "w") as settings_file:
        json.dump(new_settings, settings_file, indent=4)


def create_and_save_the_settings(test_mode_settings=False,
                                 debug_mode_settings=True):
    """
    Creates and saves settings.
    """
    global temp_json

    save_settings(temp_json)

    t_mode_settings(test_mode_settings)
    d_mode_settings(debug_mode_settings)

    return temp_json


def t_mode_settings(new_value):
    """
    Changes the test_mode setting.

    Inputs:
      * new_value: New value for the test_mode
    """

    settings = the_settings()
    settings["test_mode"] = new_value
    save_settings(settings)


def d_mode_settings(new_value):
    """
    Changes the debug_mode setting.

    Inputs:
      * new_value: New value for the debug_mode
    """

    settings = the_settings()
    settings["debug_mode"] = new_value
    save_settings(settings)


def ft_mode_settings(new_value):
    """
    Changes the funtionaltest_mode setting.

    Inputs:
      * new_value: New value for the funtionaltest_mode
    """

    settings = the_settings()
    settings["funtionaltest_mode"] = new_value
    save_settings(settings)


def mt_settings(new_value):
    """
    Changes the mute_notifications setting.

    Inputs:
      * new_value: New value for the mute_notifications
    """

    settings = the_settings()
    settings["mute_notifications"] = new_value
    save_settings(settings)


def dark_mode_settings(new_value):
    """
    Changes the dark_mode setting.

    Inputs:
      * new_value: New value for the dark_mode
    """

    settings = the_settings()
    settings["dark_mode"] = new_value
    save_settings(settings)


def publisher_mode_settings(new_value):
    """
    Changes the publisher_mode setting.

    Inputs:
      * new_value: New value for the publisher_mode
    """

    settings = the_settings()
    settings["publisher_mode"] = new_value
    save_settings(settings)


def baklava_settings(new_value):
    """
    Changes the baklava setting.

    Inputs:
      * new_value: New value for the baklava
    """

    settings = the_settings()
    settings["baklava"] = new_value
    save_settings(settings)


def change_wallet(new_value):
    """
    Changes the debug_mode setting.

    Inputs:
      * new_value: New value for the debug_mode
    """

    settings = the_settings()
    settings["wallet"] = new_value
    save_settings(settings)
    return settings["wallet"]


def the_settings():
    """
    Returns the settings. If it doesn't exist, it creates,
    saves and returns.
    """

    os.chdir(get_config()["main_folder"])

    if not os.path.exists(SETTING_PATH):
        return create_and_save_the_settings()
    with open(SETTING_PATH, "r") as settings_file:
        the_setting = json.load(settings_file)

    missing = False
    for element in temp_json:
        if element not in the_setting:
            missing = True
            the_setting[element] = temp_json[element]
    if missing:
        save_settings(the_setting)

    return the_setting



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import logging
import os
import sys
import time

from naruno.config import LOGS_PATH
from naruno.lib.config_system import get_config
from naruno.lib.settings_system import the_settings

global_logger = []

format = (
    "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
)


class ColoredFormatter(logging.Formatter):
    grey = "\x1b[38;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"

    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: grey + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset,
    }

    if sys.platform.lower() == "win32":
        os.system("")

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


def get_logger(name):
    logger = logging.getLogger(name)

    if not any(element == logger for element in global_logger):
        level = logging.DEBUG if the_settings()["debug_mode"] else logging.INFO
        logger.setLevel(level)
        # create console handler and set level to debug
        ch = logging.StreamHandler()
        ch.setLevel(level)
        # create formatter
        formatter = logging.Formatter(format)
        colored_formatter = ColoredFormatter(format)
        # add formatter to ch
        ch.setFormatter(colored_formatter)
        # add ch to logger
        logger.addHandler(ch)
        # file
        main_folder = get_config()["main_folder"]
        fh = logging.FileHandler(
            os.path.join(main_folder, LOGS_PATH, f"{name}.log"))
        fh.setLevel(level)
        fh.setFormatter(formatter)
        logger.addHandler(fh)

        global_logger.append(logger)
    return logger


def clear_logs():
    if the_settings()["disable_log_clearing"]:
        return
    main_folder = get_config()["main_folder"]
    for file in os.listdir(os.path.join(main_folder, LOGS_PATH)):
        if file.endswith(".log"):
            open(os.path.join(main_folder, LOGS_PATH, file), "w").close()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import itertools
import json
import os
import time

from naruno.config import *
from naruno.lib.config_system import get_config


class Unl:

    @staticmethod
    def save_new_unl_node(node_id):
        """
        Saves the new unl.
        """

        nodes_list = Unl.get_unl_nodes()

        already_in_list = any(element == node_id for element in nodes_list)
        if not already_in_list:

            nodes_list[node_id] = {}
            nodes_list[node_id]["date"] = time.time()

            os.chdir(get_config()["main_folder"])
            with open(UNL_NODES_PATH, "w") as unl_nodes_file:
                json.dump(nodes_list, unl_nodes_file, indent=4)

    @staticmethod
    def get_unl_nodes(custom_UNL_NODES_PATH=None):
        """
        Returns the UNL nodes list from UNL_NODES_PATH.
        """

        the_UNL_NODES_PATH = (UNL_NODES_PATH if custom_UNL_NODES_PATH is None
                              else custom_UNL_NODES_PATH)

        if not os.path.exists(the_UNL_NODES_PATH):
            return {}

        os.chdir(get_config()["main_folder"])
        with open(the_UNL_NODES_PATH, "r") as unl_nodes_file:
            return json.load(unl_nodes_file)

    @staticmethod
    def get_as_node_type(id_list):
        """
        Converts the UNL node list to Node class.
        """

        from naruno.node.server.server import server

        nodes = [] if server.Server is None else server.Server.clients
        return nodes

    @staticmethod
    def node_is_unl(node_id):
        """
        Returns the this node is unl or not.
        """

        for unl in Unl.get_unl_nodes():
            temp_unl = unl
            if node_id == temp_unl:
                return True
        return False

    @staticmethod
    def unl_node_delete(node_id):
        """
        Deletes the UNL node
        """

        saved_nodes = Unl.get_unl_nodes()
        if node_id in saved_nodes:
            del saved_nodes[node_id]

            os.chdir(get_config()["main_folder"])
            with open(UNL_NODES_PATH, "w") as connected_node_file:
                json.dump(saved_nodes, connected_node_file, indent=4)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
from hashlib import sha256

from naruno.config import COMMANDERS_PATH
from naruno.lib.config_system import get_config


def DeleteCommander(tx):
    os.chdir(get_config()["main_folder"])
    for entry in os.scandir(COMMANDERS_PATH):
        if tx in entry.name:
            os.remove(entry.path)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import json
import os
import sys
from hashlib import sha256

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
from naruno.config import SIGNS_PATH
from naruno.lib.config_system import get_config
from naruno.wallet.ellipticcurve.ecdsa import Ecdsa
from naruno.wallet.ellipticcurve.publicKey import PublicKey
from naruno.wallet.ellipticcurve.signature import Signature
from naruno.wallet.wallet_import import Address, wallet_import


def verify(path: str) -> bool:
    """
    Verifies the signature of the sign file.

    Args:
        path (str): Path of the sign file
    """
    sign_json = None
    os.chdir(get_config()["main_folder"])
    with contextlib.suppress(FileNotFoundError):
        with open(path, "r") as sign_file:
            sign_json = json.load(sign_file)

    if sign_json is None:
        return False


    result = (Ecdsa.verify(
        sign_json["data"],
        Signature.fromBase64(sign_json["signature"]),
        PublicKey.fromPem(sign_json["publickey"]),
    ), sign_json["data"], Address(sign_json["publickey"]))

    return result


if __name__ == "__main__":
    from naruno.lib.sign import sign

    print(verify(sign("Onur Atakan", "123")))



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from naruno.wallet.ellipticcurve.point import Point


class Math:

    @classmethod
    def multiply(cls, p, n, N, A, P):
        """
        Fast way to multily point and scalar in elliptic curves

        :param p: First Point to mutiply
        :param n: Scalar to mutiply
        :param N: Order of the elliptic curve
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """
        return cls._fromJacobian(
            cls._jacobianMultiply(cls._toJacobian(p), n, N, A, P), P)

    @classmethod
    def add(cls, p, q, A, P):
        """
        Fast way to add two points in elliptic curves

        :param p: First Point you want to add
        :param q: Second Point you want to add
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """
        return cls._fromJacobian(
            cls._jacobianAdd(cls._toJacobian(p), cls._toJacobian(q), A, P),
            P,
        )

    @classmethod
    def inv(cls, x, n):
        """
        Extended Euclidean Algorithm. It's the 'division' in elliptic curves

        :param x: Divisor
        :param n: Mod for division
        :return: Value representing the division
        """

        lm = 1
        hm = 0
        low = x % n
        high = n

        while low > 1:
            r = high // low
            nm = hm - lm * r
            nw = high - low * r
            high = low
            hm = lm
            low = nw
            lm = nm

        return lm % n

    @classmethod
    def _toJacobian(cls, p):
        """
        Convert point to Jacobian coordinates

        :param p: First Point you want to add
        :return: Point in Jacobian coordinates
        """
        return Point(p.x, p.y, 1)

    @classmethod
    def _fromJacobian(cls, p, P):
        """
        Convert point back from Jacobian coordinates

        :param p: First Point you want to add
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point in default coordinates
        """
        z = cls.inv(p.z, P)
        x = (p.x * z**2) % P
        y = (p.y * z**3) % P

        return Point(x, y, 0)

    @classmethod
    def _jacobianDouble(cls, p, A, P):
        """
        Double a point in elliptic curves

        :param p: Point you want to double
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """

        ysq = (p.y**2) % P
        S = (4 * p.x * ysq) % P
        M = (3 * p.x**2 + A * p.z**4) % P
        nx = (M**2 - 2 * S) % P
        ny = (M * (S - nx) - 8 * ysq**2) % P
        nz = (2 * p.y * p.z) % P

        return Point(nx, ny, nz)

    @classmethod
    def _jacobianAdd(cls, p, q, A, P):
        """
        Add two points in elliptic curves

        :param p: First Point you want to add
        :param q: Second Point you want to add
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """

        U1 = (p.x * q.z**2) % P
        U2 = (q.x * p.z**2) % P
        S1 = (p.y * q.z**3) % P
        S2 = (q.y * p.z**3) % P

        H = U2 - U1
        R = S2 - S1
        H2 = (H * H) % P
        H3 = (H * H2) % P
        U1H2 = (U1 * H2) % P
        nx = (R**2 - H3 - 2 * U1H2) % P
        ny = (R * (U1H2 - nx) - S1 * H3) % P
        nz = (H * p.z * q.z) % P

        return Point(nx, ny, nz)

    @classmethod
    def _jacobianMultiply(cls, p, n, N, A, P):
        """
        Multily point and scalar in elliptic curves

        :param p: First Point to mutiply
        :param n: Scalar to mutiply
        :param N: Order of the elliptic curve
        :param P: Prime number in the module of the equation Y^2 = X^3 + A*X + B (mod p)
        :param A: Coefficient of the first-order term of the equation Y^2 = X^3 + A*X + B (mod p)
        :return: Point that represents the sum of First and Second Point
        """
        if n == 1:
            return p

        if (n % 2) == 0:
            return cls._jacobianDouble(
                cls._jacobianMultiply(p, n // 2, N, A, P), A, P)

        return cls._jacobianAdd(
            cls._jacobianDouble(cls._jacobianMultiply(p, n // 2, N, A, P), A,
                                P),
            p,
            A,
            P,
        )



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.blockchain.block.block_main import Block
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.node.unl import Unl


def GetCandidateBlocks(custom_nodes_list=None, block: Block = None):
    """
    Collects candidate blocks and candidate block hashes
    from connected unl nodes and returns them in the
    candidate_block class
    """

    nodes = (Unl.get_as_node_type(Unl.get_unl_nodes())
             if custom_nodes_list is None else custom_nodes_list)

    the_candidate_blocks = []
    the_candidate_block_hashes = []

    for node in nodes:
        if node.candidate_block is not None:
            if int(node.candidate_block["sequence_number"]
                   ) == block.sequence_number+block.empty_block_number:
                the_candidate_blocks.append(node.candidate_block)
            else:
                for i in node.candidate_block_history:
                    if i["sequence_number"] == block.sequence_number+block.empty_block_number:
                        the_candidate_blocks.append(i)
        else:
            pass
        if node.candidate_block_hash is not None:
            if (int(node.candidate_block_hash["sequence_number"]) ==
                    block.sequence_number+block.empty_block_number):
                the_candidate_block_hashes.append(node.candidate_block_hash)
            else:
                for i in node.candidate_block_hash_history:
                    if i["sequence_number"] == block.sequence_number+block.empty_block_number:
                        the_candidate_block_hashes.append(i)
        else:
            pass

    if block is not None:
        new_list = []

        signature_list = []

        a_time = "self"

        for element in block.validating_list:
            new_list.append(element.dump_json())
            signature_list.append(element.signature)
        the_candidate_blocks.append({
            "action": "myblock",
            "transaction": new_list,
            "signature": a_time,
            "sequence_number": block.sequence_number+block.empty_block_number,
        })

        the_candidate_block_hashes.append({
            "action":
            "myblockhash",
            "hash":
            block.hash,
            "previous_hash":
            block.previous_hash,
            "signature":
            a_time,
            "sequence_number":
            block.sequence_number+block.empty_block_number,
        })

    not_none_the_candidate_blocks = []

    for none_candidate_block in the_candidate_block_hashes:
        if not none_candidate_block["hash"] == None:
            not_none_the_candidate_blocks.append(none_candidate_block)

    return candidate_block(the_candidate_blocks, not_none_the_candidate_blocks)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from binascii import hexlify
from binascii import unhexlify
from sys import version_info as pyVersion

if pyVersion.major == 3:
    # py3 constants and conversion functions

    stringTypes = (str, )
    intTypes = (int, float)

    def toString(string, encoding="utf-8"):
        return string.decode(encoding)

    def toBytes(string, encoding="utf-8"):
        return string.encode(encoding)

    def safeBinaryFromHex(hexadecimal):

        return unhexlify(hexadecimal)

    def safeHexFromBinary(byteString):
        return toString(hexlify(byteString))



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import json
import os

from naruno.blockchain.block.block_main import Block
from naruno.config import TEMP_BLOCK_PATH
from naruno.consensus.rounds.round_1.process.transactions.checks.duplicated import Remove_Duplicates
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger

logger = get_logger("BLOCKCHAIN")


def GetBlock(custom_TEMP_BLOCK_PATH=None, get_normal_block=False, dont_clean=False):
    """
    Returns the block.
    """
    logger.info("Getting block from disk")

    the_TEMP_BLOCK_PATH = (TEMP_BLOCK_PATH if custom_TEMP_BLOCK_PATH is None
                           else custom_TEMP_BLOCK_PATH)

    os.chdir(get_config()["main_folder"])

    highest_the_TEMP_BLOCK_PATH = the_TEMP_BLOCK_PATH
    highest_number = 0

    highest_second_number = 0
    highest_other_high_number = 0
    for file in os.listdir("db/"):
        if ("db/" + file).startswith(the_TEMP_BLOCK_PATH) and not ("db/" + file) == the_TEMP_BLOCK_PATH:
            number = int((("db/" + file).replace(the_TEMP_BLOCK_PATH, "")).split("-")[1]) #seq
            high_number = int((("db/" + file).replace(the_TEMP_BLOCK_PATH, "")).split("-")[3]) #val
            other_high_number = int((("db/" + file).replace(the_TEMP_BLOCK_PATH, "")).split("-")[2]) #val

            # Write a code for getting the blocks with high number
            if number > highest_number:
                highest_number = number
                highest_the_TEMP_BLOCK_PATH = "db/" + file
                highest_second_number = high_number
                highest_other_high_number = other_high_number
            elif number == highest_number:
                if high_number > highest_second_number:
                    highest_number = number
                    highest_the_TEMP_BLOCK_PATH = "db/" + file
                    highest_second_number = high_number
                    highest_other_high_number = other_high_number
                elif high_number == highest_second_number:
                    if other_high_number > highest_other_high_number:
                        highest_number = number
                        highest_the_TEMP_BLOCK_PATH = "db/" + file
                        highest_second_number = high_number
                        highest_other_high_number = other_high_number
            
                


    logger.debug("Highest block: " + highest_the_TEMP_BLOCK_PATH)


    result_normal = Block("non")

    with contextlib.suppress(json.decoder.JSONDecodeError):
        with open(the_TEMP_BLOCK_PATH, "r") as block_file:
            the_block_json = json.load(block_file)
            result_normal = Block.load_json(the_block_json)


    with open(highest_the_TEMP_BLOCK_PATH, "r") as block_file:
        the_block_json = json.load(block_file)
        result_highest = Block.load_json(the_block_json)

    result_normal = Remove_Duplicates(result_normal)
    result_highest = Remove_Duplicates(result_highest)

    result_normal.validating_list = sorted(result_normal.validating_list,
                                   key=lambda x: x.fromUser)    

    result_highest.validating_list = sorted(result_highest.validating_list,
                                   key=lambda x: x.fromUser)    

    if get_normal_block:
        return result_normal

    if result_normal.sequence_number + result_normal.empty_block_number > result_highest.sequence_number + result_highest.empty_block_number:
        return result_normal
    elif result_normal.sequence_number + result_normal.empty_block_number == result_highest.sequence_number + result_highest.empty_block_number:

        result_normal_situation = 0
        result_highest_situation = 0
        if result_normal.round_1:
            result_normal_situation += 1
        if result_normal.round_2:
            result_normal_situation += 1

        if result_highest.round_1:
            result_highest_situation += 1
        if result_highest.round_2:
            result_highest_situation += 1


        if len(result_normal.validating_list) > len(result_highest.validating_list):

                return result_normal
        else:

                return result_highest

        
    else:
        return result_highest





#!/usr/bin/python3
# -*- coding: utf-8 -*-
from naruno.wallet.ellipticcurve.utils.binary import hexFromInt
from naruno.wallet.ellipticcurve.utils.binary import intFromHex


def oidFromHex(hexadecimal):
    firstByte, remainingBytes = hexadecimal[:2], hexadecimal[2:]
    firstByteInt = intFromHex(firstByte)
    oid = [firstByteInt // 40, firstByteInt % 40]
    oidInt = 0
    while len(remainingBytes) > 0:
        byte, remainingBytes = remainingBytes[:2], remainingBytes[2:]
        byteInt = intFromHex(byte)
        if byteInt >= 128:
            oidInt = (128 * oidInt) + (byteInt - 128)
            continue
        oidInt = (128 * oidInt) + byteInt
        oid.append(oidInt)
        oidInt = 0
    return oid


def oidToHex(oid):
    hexadecimal = hexFromInt(40 * oid[0] + oid[1])
    for number in oid[2:]:
        hexadecimal += _oidNumberToHex(number)
    return hexadecimal


def _oidNumberToHex(number):
    hexadecimal = ""
    endDelta = 0
    while number > 0:
        hexadecimal = hexFromInt((number % 128) + endDelta) + hexadecimal
        number //= 128
        endDelta = 128
    return hexadecimal or "00"



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from datetime import datetime

from naruno.wallet.ellipticcurve.utils.binary import bitsFromHex
from naruno.wallet.ellipticcurve.utils.binary import byteStringFromHex
from naruno.wallet.ellipticcurve.utils.binary import hexFromInt
from naruno.wallet.ellipticcurve.utils.binary import intFromHex
from naruno.wallet.ellipticcurve.utils.oid import oidFromHex
from naruno.wallet.ellipticcurve.utils.oid import oidToHex


class DerFieldType:

    integer = "integer"
    bitString = "bitString"
    octetString = "octetString"
    null = "null"
    object = "object"
    printableString = "printableString"
    utcTime = "utcTime"
    sequence = "sequence"
    set = "set"
    oidContainer = "oidContainer"
    publicKeyPointContainer = "publicKeyPointContainer"


_hexTagToType = {
    "02": DerFieldType.integer,
    "03": DerFieldType.bitString,
    "04": DerFieldType.octetString,
    "05": DerFieldType.null,
    "06": DerFieldType.object,
    "13": DerFieldType.printableString,
    "17": DerFieldType.utcTime,
    "30": DerFieldType.sequence,
    "31": DerFieldType.set,
    "a0": DerFieldType.oidContainer,
    "a1": DerFieldType.publicKeyPointContainer,
}
_typeToHexTag = {v: k for k, v in _hexTagToType.items()}


def encodeConstructed(*encodedValues):
    return encodePrimitive(DerFieldType.sequence, "".join(encodedValues))


def encodePrimitive(tagType, value):
    if tagType == DerFieldType.integer:
        value = _encodeInteger(value)
    if tagType == DerFieldType.object:
        value = oidToHex(value)
    return "{tag}{size}{value}".format(tag=_typeToHexTag[tagType],
                                       size=_generateLengthBytes(value),
                                       value=value)


def parse(hexadecimal):
    if not hexadecimal:
        return []
    typeByte, hexadecimal = hexadecimal[:2], hexadecimal[2:]
    length, lengthBytes = _readLengthBytes(hexadecimal)
    content, hexadecimal = (
        hexadecimal[lengthBytes:lengthBytes + length],
        hexadecimal[lengthBytes + length:],
    )

    tagData = _getTagData(typeByte)
    if tagData["isConstructed"]:
        content = parse(content)

    valueParser = {
        DerFieldType.object: _parseOid,
        DerFieldType.integer: _parseInteger,
    }.get(tagData["type"], _parseAny)
    return [valueParser(content)] + parse(hexadecimal)


def _parseAny(hexadecimal):
    return hexadecimal


def _parseOid(hexadecimal):
    return tuple(oidFromHex(hexadecimal))


def _parseInteger(hexadecimal):
    integer = intFromHex(hexadecimal)
    bits = bitsFromHex(hexadecimal[0])
    if bits[0] == "0":  # negative numbers are encoded using two's complement
        return integer


def _encodeInteger(number):
    hexadecimal = hexFromInt(abs(number))

    bits = bitsFromHex(hexadecimal[0])
    if (
            bits[0] == "1"
    ):  # if first bit was left as 1, number would be parsed as a negative integer with two's complement
        hexadecimal = "00" + hexadecimal
    return hexadecimal


def _readLengthBytes(hexadecimal):
    lengthBytes = 2
    lengthIndicator = intFromHex(hexadecimal[0:lengthBytes])
    isShortForm = (lengthIndicator < 128
                   )  # checks if first bit of byte is 1 (a.k.a. short-form)
    if isShortForm:
        length = lengthIndicator * 2
        return length, lengthBytes


def _generateLengthBytes(hexadecimal):
    size = len(hexadecimal) // 2
    length = hexFromInt(size)
    # checks if first bit of byte should be 0 (a.k.a. short-form flag)
    if size < 128:
        return length.zfill(2)


def _getTagData(tag):
    bits = bitsFromHex(tag)
    bit8, bit7, bit6 = bits[:3]

    tagClass = {
        "0": {
            "0": "universal",
            "1": "application",
        },
        "1": {
            "0": "context-specific",
            "1": "private",
        },
    }[bit8][bit7]
    isConstructed = bit6 == "1"

    return {
        "class": tagClass,
        "isConstructed": isConstructed,
        "type": _hexTagToType.get(tag),
    }



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from naruno.config import *
from naruno.lib.config_system import get_config


def get_saved_wallet():

    os.chdir(get_config()["main_folder"])

    if not os.path.exists(WALLETS_PATH):
        return {}

    with open(WALLETS_PATH, "r") as wallet_list_file:
        return json.load(wallet_list_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import time

from kivy.app import App
from kivymd.uix.button import MDFlatButton
from kivymd.uix.button import MDRaisedButton
from kivymd.uix.gridlayout import MDGridLayout
from kivymd.uix.screen import MDScreen
from kivymd_extensions.sweetalert import SweetAlert

import naruno.gui.the_naruno_gui_app
from naruno.lib.settings_system import d_mode_settings
from naruno.lib.settings_system import dark_mode_settings
from naruno.lib.settings_system import mt_settings
from naruno.lib.settings_system import t_mode_settings
from naruno.lib.settings_system import the_settings


class SettingsScreen(MDScreen):
    pass


class SettingsBox(MDGridLayout):
    cols = 2

    d_first_status = the_settings()["debug_mode"]
    t_first_status = the_settings()["test_mode"]
    mt_first_status = the_settings()["mute_notifications"]
    dark_mode_first_status = the_settings()["dark_mode"]

    def D_Status_Changing(self, instance, value):
        d_mode_settings(value)

    def T_Status_Changing(self, instance, value):
        t_mode_settings(value)

    def MT_Status_Changing(self, instance, value):
        mt_settings(value)

    def show_dialog(self):
        button_ok = MDRaisedButton(
            text="OK",
            font_size=16,
            on_release=self.callback,
        )
        button_cancel = MDFlatButton(
            text="CANCEL",
            font_size=16,
            on_release=self.callback,
        )
        self.alert = SweetAlert()
        self.alert.fire(
            "The app will restart.",
            buttons=[button_ok, button_cancel],
            type="info",
        )

    def callback(self, instance_button):
        if instance_button.text == "OK":
            self.alert.dismiss()
            SettingsBox.dark_mode_first_status = the_settings()["dark_mode"]
            naruno.gui.the_naruno_gui_app.the_naruno_gui.restart(
            )

        else:
            self.alert.dismiss()

    def DARK_MODE_Status_Changing(self, instance, value):

        dark_mode_settings(value)
        self.show_dialog()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import GetBlockshash
from naruno.blockchain.block.blocks_hash import GetBlockshash_part
from naruno.blockchain.block.blocks_hash import SaveBlockshash
from naruno.blockchain.block.blocks_hash import SaveBlockshash_part
from naruno.blockchain.block.hash.calculate_hash import CalculateHash
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.rounds.round_1.checks.checks_main import \
    round_check
from naruno.lib.log import get_logger
from naruno.node.get_candidate_blocks import GetCandidateBlocks
from naruno.node.server.server import server
from naruno.node.unl import Unl
from naruno.transactions.get_transaction import GetTransaction
from naruno.transactions.process_the_transaction import \
    ProccesstheTransaction

logger = get_logger("CONSENSUS_FIRST_ROUND")


def find_validated(block: Block, candidate_class: candidate_block,
                   unl_nodes: dict) -> list:
    logger.info("Finding process of validating list is started.")
    temp_validating_list = []
    logger.debug(f"First temp_validating_list: {temp_validating_list}")
    for candidate_block in candidate_class.candidate_blocks[:]:
        logger.debug(f"Candidate block {candidate_block}")

        for other_block_tx in candidate_block["transaction"]:

            tx_valid = 1

            if len(candidate_class.candidate_blocks) != 1:

                for other_block in candidate_class.candidate_blocks[:]:
                    if candidate_block["signature"] != other_block["signature"]:

                        for other_block_txs in other_block["transaction"]:
                            if other_block_tx.signature == other_block_txs.signature:

                                tx_valid += 1
            else:
                tx_valid += 1


            logger.debug(
                f"Tx valid of {other_block_tx.signature} : {tx_valid}")
            if tx_valid > ((len(unl_nodes) + 1) / 2):

                already_in_ok = False
                for alrady_tx in temp_validating_list[:]:

                    if other_block_tx.signature == alrady_tx.signature:
                        logger.debug(
                            "The transaction is already in the list")
                        already_in_ok = True
                if not already_in_ok:
                    logger.debug(
                        f"Transaction is valid ({other_block_tx.signature})")
                    temp_validating_list.append(other_block_tx)
    logger.debug(f"First temp_validating_list: {temp_validating_list}")
    return temp_validating_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import sys

import qrcode
from qrcode.image.styledpil import StyledPilImage
from qrcode.image.styles.colormasks import SolidFillColorMask

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))

from naruno.config import QR_CODE_PATH
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from hashlib import sha256


logger = get_logger("LIB")


def qr(data):
    logger.info("Qr code generator is started")
    logger.debug(f"data: {data}")
    main_folder = get_config()["main_folder"]
    data_sha256 = sha256(data.encode("utf-8")).hexdigest()
    location = f"{main_folder}/{QR_CODE_PATH}{data_sha256}.png"
    

    if not os.path.exists(location):
        logger.info("Qr code is not exist, it will be created")
        qr = qrcode.QRCode(error_correction=qrcode.constants.ERROR_CORRECT_H)
        qr.add_data(data)
        qr.make(fit=True)
        icon = f"{main_folder}/gui_lib/images/logo_sm_orb_fw.png"
        logger.debug(f"icon: {icon}")
        qr_img = qr.make_image(
            image_factory=StyledPilImage,
            embeded_image_path=icon,
            color_mask=SolidFillColorMask(front_color=(92, 107, 192)),
        )
        qr_img.save(location)
    else:
        logger.info("Qr code already exists")

    logger.info(f"location: {location}")
    logger.info("Qr code generator is finished.")
    return location

if __name__ == "__main__":
    qr("Ali_Eren_TABAK")


#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import json
import os

from naruno.config import MY_TRANSACTION_PATH
from naruno.lib.config_system import get_config
from naruno.transactions.transaction import Transaction


def GetMyTransaction(sended=None, validated=None, turn_json=False) -> list:
    """
    Returns the transaction db.
    """

    os.chdir(get_config()["main_folder"])

    if not os.path.exists(MY_TRANSACTION_PATH):
        return []

    the_transactions = []

    # find the my transaction folder with os scandir
    for entry in os.scandir(MY_TRANSACTION_PATH):
        if (entry.name != "README.md"
                and not entry.name.startswith("validated")
                and not entry.name.startswith("sended")):
            try:
                the_transactions_json = json.load(open(entry.path, "r"))
                # Find "validatedentry.name" and "sendedentry.name" files
                each_validated = False
                each_sended = False
                if os.path.exists(
                        os.path.join(MY_TRANSACTION_PATH,
                                    "validated" + entry.name)):
                    each_validated = True
                if os.path.exists(
                        os.path.join(MY_TRANSACTION_PATH, "sended" + entry.name)):
                    each_sended = True
                the_transactions.append([
                    Transaction.load_json(the_transactions_json),
                    each_validated,
                    each_sended,
                ])
            except json.decoder.JSONDecodeError:
                with contextlib.suppress(Exception):
                    os.remove(entry.path)

    if sended is not None:
        the_transactions = [tx for tx in the_transactions if tx[2] == sended]

    if validated is not None:
        the_transactions = [
            tx for tx in the_transactions if tx[1] == validated
        ]

    #sort
    the_transactions.sort(key=lambda x: x[0].signature)

    if turn_json:
        the_transactions = {
            tx[0].signature: {
                "transaction": tx[0].dump_json(),
                "validated": tx[1],
                "sended": tx[2],
            }
            for tx in the_transactions
        }



    return the_transactions



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from kivy.properties import ColorProperty
from kivy.properties import StringProperty
from kivymd.theming import ThemableBehavior
from kivymd.uix.behaviors import RectangularRippleBehavior
from kivymd.uix.boxlayout import MDBoxLayout
from kivymd.uix.screen import MDScreen

import naruno.gui.the_naruno_gui_app
from naruno.gui.popup import popup


class NarunoRootScreen(MDScreen):

    def close_app(self, widget=None):
        self.close_app_dialog = popup(
            text="Are you sure you want to close ?",
            target=naruno.gui.the_naruno_gui_app.
            the_naruno_gui.stop,
            type="question",
        )


class NarunoListItem(ThemableBehavior, RectangularRippleBehavior,
                       MDBoxLayout):
    text = StringProperty()
    secondary_text = StringProperty()
    tertiary_text = StringProperty()
    bar_color = ColorProperty((1, 0, 0, 1))


class NarunoSeeAllButton(RectangularRippleBehavior, MDBoxLayout):
    pass



import os
import shutil
import sys
import time
from typing import Union

sys.path.append(os.path.join(os.path.dirname(__file__), "..", "..", ".."))
from naruno.config import BACKUPS_PATH
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger

logger = get_logger("LIB")


def naruno_export() -> Union[str, None]:
    """
    Create a ZIP archive of the `db` folder in the main directory of the application.

    Returns:
        The path of the created ZIP file, or None if the ZIP file could not be created.
    """
    logger.info("Export system is started")
    main_folder = get_config()["main_folder"]
    zip_dir = f"{main_folder}/db/"
    result_file = f"{main_folder}/{BACKUPS_PATH}{int(time.time())}"
    logger.debug(f"zip_dir: {zip_dir}")
    logger.debug(f"result_file: {result_file}.zip")
    shutil.make_archive(result_file, "zip", zip_dir)
    logger.info("Export complated")

    return f"{result_file}.zip"


if __name__ == "__main__":
    naruno_export()



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os
from hashlib import sha256

from kivy.core.clipboard import Clipboard
from kivymd.uix.bottomsheet import MDListBottomSheet
from kivymd.uix.button import MDFlatButton
from kivymd.uix.gridlayout import MDGridLayout
from kivymd.uix.screen import MDScreen
from kivymd_extensions.sweetalert import SweetAlert

import naruno.gui.the_naruno_gui_app
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.config import MY_TRANSACTION_EXPORT_PATH
from naruno.gui.popup import popup
from naruno.lib.export import export_the_transactions
from naruno.lib.settings_system import the_settings
from naruno.lib.sign import sign
from naruno.lib.verify import verify
from naruno.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction
from naruno.transactions.my_transactions.save_to_my_transaction import \
    SavetoMyTransaction
from naruno.transactions.send import send
from naruno.wallet.wallet_import import wallet_import


class OperationScreen(MDScreen):
    pass


class OperationBox(MDGridLayout):
    cols = 2

    def sent_the_coins(self):
        the_block = GetBlock()

        if (float(self.send_coin_dialog.input_results["Amount"]) >=
                the_block.minumum_transfer_amount):
            if (wallet_import(int(the_settings()["wallet"]), 2) == sha256(
                    self.send_coin_dialog.input_results["Password"].encode(
                        "utf-8")).hexdigest()):
                block = the_block
                send_tx = send(
                    self.send_coin_dialog.input_results["Password"],
                    self.send_coin_dialog.input_results["Receiver"],
                    amount=float(
                        self.send_coin_dialog.input_results["Amount"]),
                    data=str(self.send_coin_dialog.input_results["Data"]),
                    block=block,
                )
                if send_tx != False:

                    from naruno.node.server.server import server

                    if server.Server is None:
                        popup(title="Please start the node server",
                              type="failure")
                        return False

                    SavetoMyTransaction(send_tx, sended=True)
                    server.send_transaction(send_tx)
                    SaveBlock(block)
            else:
                popup(title="Password is not correct", type="failure")

    def show_send_coin_dialog(self):
        self.send_coin_dialog = popup(
            title="Send Coin",
            target=self.sent_the_coins,
            inputs=[
                ["Receiver", False],
                ["Amount", False],
                ["Data", False],
                ["Password", True],
            ],
        )

    def sign_the_data(self):
        path = sign(
            self.sign_dialog.input_results["Data"],
            self.sign_dialog.input_results["Password"],
        )
        if path == "None":
            popup(title="Password is not correct", type="failure")
        else:
            Clipboard.copy(path)
            popup(
                title="Signed data file created",
                text="The file has been copied to your clipboard.",
                thirdly_title=path,
                type="success",
            )

    def show_sign_dialog(self):
        self.sign_dialog = popup(
            title="Sign Data",
            target=self.sign_the_data,
            inputs=[["Data", False], ["Password", True]],
        )

    def verify_the_data(self):
        result = verify(self.verify_dialog.input_results["Path"])

        if result[0] == True:
            data_text = f"{result[1][:20]}..." if len(
                result[1]) > 20 else result[1]
            popup(
                title="Data is verified",
                text=f"The data is : {data_text}",
                thirdly_title=f"The sender is : {result[2]}",
                type="success",
            )
        else:
            popup(title="Data is not verified", type="failure")

    def show_verify_dialog(self):
        self.verify_dialog = popup(
            title="Verify Signed Data",
            target=self.verify_the_data,
            inputs=[["Path", False]],
        )

    def send_coin(self):
        try:
            GetBlock()
        except FileNotFoundError:
            popup(title="Please connect to an network.", type="failure")
            return False
        self.show_send_coin_dialog()

    def sign(self):
        self.show_sign_dialog()

    def verify(self):
        self.show_verify_dialog()

    def export_transaction_csv(self):
        if export_the_transactions():
            Clipboard.copy(MY_TRANSACTION_EXPORT_PATH)
            popup(
                title=
                f"CSV file created in {MY_TRANSACTION_EXPORT_PATH} directory, The directory has been copied to your clipboard.",
                type="success",
            )

        else:
            popup(title="You have not a transaction", type="warning")

    def callback_for_transaction_history_items(self, *args):
        the_signature_of_tx = args[0][:96]
        Clipboard.copy(the_signature_of_tx)
        popup(
            title=
            "The signature of transaction has been copied to your clipboard.",
            text=f"The signature is : {the_signature_of_tx}",
            type="success",
        )

    def transaction_history(self):
        transactions = GetMyTransaction()
        if len(transactions) != 0:
            bottom_sheet_menu = MDListBottomSheet(radius=25, radius_from="top")
            data = {
                tx[0]:
                f"{tx[0].toUser} | {str(tx[0].amount)} | {str(tx[0].transaction_fee)} | {str(tx[1])}"
                for tx in transactions
            }

            for item in data.items():
                bottom_sheet_menu.add_item(
                    item[1],
                    lambda x, y=item[0]: self.
                    callback_for_transaction_history_items(y),
                )
            bottom_sheet_menu.open()
        else:
            popup(title="You have not a transaction", type="warning")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.accounts.get_accounts import GetAccounts
from naruno.transactions.pending.get_pending import GetPending
from naruno.wallet.wallet_import import Address


def GetSequanceNumber(user, account_list=None, dont_convert=False, block=None):
    user = Address(user) if not dont_convert else user
    sequence_number = 0
    the_account_list = GetAccounts() if account_list is None else account_list
    the_account_list.execute(
        f"SELECT * FROM account_list WHERE address = '{user}'")
    for Accounts in the_account_list.fetchall():
        sequence_number = Accounts[1]
        break
    if block is not None:
        if not block.just_one_tx:
            for tx in block.validating_list + GetPending():
                if Address(tx.fromUser) == user:
                    sequence_number += 1
    return sequence_number



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))
import argparse

os.environ["KIVY_NO_ARGS"] = "1"
from kivy import Config
from kivy.core.window import Window
from kivy.lang import Builder
from kivymd.app import MDApp

import naruno.gui.the_naruno_gui_app
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from naruno.lib.safety import safety_check
from naruno.lib.settings_system import the_settings

Config.set("graphics", "width", "700")
Config.set("graphics", "height", "450")
Config.set("graphics", "minimum_width", "700")
Config.set("graphics", "minimum_height", "450")
Config.set("input", "mouse", "mouse,disable_multitouch")

os.environ["NARUNO_ROOT"] = get_config()["main_folder"]

KV_DIR = f"{os.environ['NARUNO_ROOT']}/gui_lib/libs/kv/"

for kv_file in os.listdir(KV_DIR):
    with open(os.path.join(KV_DIR, kv_file), encoding="utf-8") as kv:
        Builder.load_string(kv.read())

KV = """
#:import FadeTransition kivy.uix.screenmanager.FadeTransition
#:import NarunoWelcomeScreen naruno.gui_lib.libs.baseclass.welcome_screen.NarunoWelcomeScreen
#:import NarunoRootScreen naruno.gui_lib.libs.baseclass.root_screen.NarunoRootScreen

ScreenManager:
    transition: FadeTransition()

    NarunoWelcomeScreen:
        name: "naruno register screen"

    NarunoRootScreen:
        name: "naruno root screen"

"""

logger = get_logger("GUI")



class GUI(MDApp):
    """
    An MDApp based  GUI class.
    GUI class consists of 2 elements:
      * title: Title of the app.
      * icon: icon of the app.
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.title = "Naruno"
        self.icon = f"{os.environ['NARUNO_ROOT']}/gui_lib/images/logo.ico"

    def build(self):
        """
        Some configurations.
        """
        Window.borderless = True
        value = the_settings()["dark_mode"]


        for i in self.theme_cls.colors["Yellow"]:
            self.theme_cls.colors["Yellow"][i] = "#5C6BC0"
        for i in self.theme_cls.colors["Dark"]:
            self.theme_cls.colors["Dark"][i] = "#303030"

        self.theme_cls.theme_style = "Dark" if value else "Light"
        self.theme_cls.primary_palette = "Yellow"  # "Purple", "Red"

        self.FONT_PATH = os.path.join(os.environ["NARUNO_ROOT"], "gui_lib",
                                      "fonts")

        self.theme_cls.font_styles.update({
            "H1":
            [os.path.join(self.FONT_PATH, "Poppins-Light"), 96, False, -1.5],
            "H2":
            [os.path.join(self.FONT_PATH, "Poppins-Light"), 60, False, -0.5],
            "H3":
            [os.path.join(self.FONT_PATH, "Poppins-Regular"), 48, False, 0],
            "H4": [
                os.path.join(self.FONT_PATH, "Poppins-Regular"),
                34,
                False,
                0.25,
            ],
            "H5":
            [os.path.join(self.FONT_PATH, "Poppins-Regular"), 24, False, 0],
            "H6":
            [os.path.join(self.FONT_PATH, "Poppins-Bold"), 20, False, 0.15],
            "Subtitle1": [
                os.path.join(self.FONT_PATH, "Poppins-Regular"),
                16,
                False,
                0.15,
            ],
            "Subtitle2": [
                os.path.join(self.FONT_PATH, "Poppins-Medium"),
                14,
                False,
                0.1,
            ],
            "Body1": [
                os.path.join(self.FONT_PATH, "Poppins-Regular"),
                16,
                False,
                0.5,
            ],
            "Body2": [
                os.path.join(self.FONT_PATH, "Poppins-Light"),
                14,
                False,
                0.25,
            ],
            "Button": [
                os.path.join(self.FONT_PATH, "Poppins-Bold"),
                14,
                True,
                1.25,
            ],
            "Caption": [
                os.path.join(self.FONT_PATH, "Poppins-Regular"),
                12,
                False,
                0.4,
            ],
            "Overline": [
                os.path.join(self.FONT_PATH, "Poppins-Regular"),
                10,
                True,
                1.5,
            ],
        })

        return Builder.load_string(KV)

    def restart(self):
        self.root.clear_widgets()
        self.stop()
        naruno.gui.the_naruno_gui_app.the_naruno_gui = (
            GUI())
        return (naruno.gui.the_naruno_gui_app.
                the_naruno_gui.run())


def arguments():
    """
    This function parses the arguments and makes the directions.
    """

    parser = argparse.ArgumentParser(
        description=
        "Naruno is a lightning-fast, secure, and scalable blockchain that is able to create transaction proofs and verification via raw data and timestamp. We remove the archive nodes and lazy web3 integrations. With Naruno everyone can get the proof (5-10MB) of their transactions via their nodes and after everyone can use in another node for verification the raw data and timestamp. Also you can integrate your web3 applications with 4 code lines (just python for now) via our remote app system. Use the menu (-m) or GUI to gain full control and use the node, operation, etc."
    )

    parser.add_argument(
        "-i",
        "--interface",
        type=str,
        help="Interface",
    )

    parser.add_argument(
        "-t",
        "--timeout",
        type=int,
        help="Timeout",
    )

    args = parser.parse_args()

    safety_check(args.interface, args.timeout)

    naruno.gui.the_naruno_gui_app.the_naruno_gui = GUI(
    )

    naruno.gui.the_naruno_gui_app.the_naruno_gui.run(
    )


def start():
    """
    Start the GUI mode.
    """

    logger.info("Starting GUI mode")
    arguments()


if __name__ == "__main__":
    start()



#!/usr/bin/python3
# -*- coding: utf-8 -*-


class File:

    @classmethod
    def read(cls, path, mode="r"):
        with open(path, mode) as blob:
            content = blob.read()
        return content



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
from base64 import b64decode
from base64 import b64encode
from binascii import hexlify
from binascii import unhexlify
from hashlib import sha256
from random import SystemRandom
from sys import version_info as pyVersion

from naruno.config import *
from naruno.lib.config_system import get_config
from naruno.lib.encryption import decrypt
from naruno.lib.encryption import encrypt
from naruno.lib.settings_system import the_settings
from naruno.wallet.ellipticcurve.privateKey import PrivateKey
from naruno.wallet.get_saved_wallet import get_saved_wallet
from naruno.wallet.save_wallet_list import save_to_wallet_list
from naruno.wallet.wallet_create import wallet_create


def wallet_import(wallet, mode, password=None):
    """
    A function for get info about a wallet.

    Inputs:
      * account: Account index of saved accounts (if you give -1 the default wallet will use)
      * mode: Information mode [0 = Public key | 1 = Private key (needs password) | 2 = Returns sha256 of password | 3 = Returns the address of account]
      * password: Some function needed password for operation you can give with this input
    """

    account = wallet

    temp_saved_wallet = get_saved_wallet()

    number_of_wallet = len(temp_saved_wallet)
    if not number_of_wallet:
        wallet_create("123")
        temp_saved_wallet = get_saved_wallet()

    if isinstance(wallet, int):
        if wallet == -1:
            account = list(temp_saved_wallet)[the_settings()["wallet"]]

        elif wallet <= (len(temp_saved_wallet) - 1):
            account = list(temp_saved_wallet)[account]
        else:
            return False
    if mode == 0:
        return temp_saved_wallet[account]["publickey"]
    elif mode == 1:
        if password is None:
            if list(temp_saved_wallet).index(account) != 0:
                return False
            my_private_key = temp_saved_wallet[account]["privatekey"]
            return my_private_key

        elif list(temp_saved_wallet).index(account) != 0:

            return decrypt(temp_saved_wallet[account]["privatekey"], password)
        else:
            if wallet != -1:
                return False
            my_private_key = temp_saved_wallet[account]["privatekey"]
            return my_private_key
    elif mode == 2:
        return temp_saved_wallet[account]["password_sha256"]

    elif mode == 3:
        my_address = temp_saved_wallet[account]["publickey"]
        my_address = "".join([
            l.strip() for l in my_address.splitlines()
            if l and not l.startswith("-----")
        ])
        my_address = Address(my_address)
        return my_address
    else:
        return False


def wallet_import_all(mode, password=None):
    temp_saved_wallet = get_saved_wallet()
    the_result = []

    for account in temp_saved_wallet:
        the_result.append(wallet_import(account, mode, password))
    return the_result


def Address(publickey):
    the_public_key = "".join([
        l.strip() for l in publickey.splitlines()
        if l and not l.startswith("-----")
    ])
    return sha256(
        sha256(the_public_key.encode("utf-8")).hexdigest().encode(
            "utf-8")).hexdigest()[-40:]



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import hashlib
import json


class Account:
    """
    The account Class.

    Account class consists of 3 elements:
      * PublicKey: Public key created with ECDSA.
      * sequence_number: Makes each transaction single use,
        incrementing 1 after each transaction.
      * balance: The amount of coins of account.
    """

    def __init__(self, PublicKey, balance, sequence_number=0):
        self.Address = PublicKey

        self.sequence_number = sequence_number
        self.balance = balance

    def get_hash(self, encoding="ascii"):
        """
        Returns sha256 created with all the elements of the account.
        """
        account_data = json.dumps(self.dump_json()).encode(encoding)
        return hashlib.sha256(account_data).hexdigest()

    def dump_json(self):
        """
        Converts and returns the account's elements to json,
        which can then be loaded with the load_json () function.
        """
        return {
            "address": self.Address,
            "balance": self.balance,
            "sequence_number": self.sequence_number,
        }

    @staticmethod
    def load_json(data):
        """
        Json returns data in the appropriate format as the account class.
        """
        return Account(data["address"], data["balance"], data["sequence_number"])

    def __str__(self):
        return self.Address



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import os
import shutil
import time

from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import GetBlockshash
from naruno.blockchain.block.blocks_hash import GetBlockshash_part
from naruno.blockchain.block.blocks_hash import SaveBlockshash
from naruno.blockchain.block.blocks_hash import SaveBlockshash_part
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.block.save_block_to_blockchain_db import \
    SaveBlockstoBlockchainDB
from naruno.config import BLOCKS_PATH
from naruno.config import TEMP_ACCOUNTS_PATH
from naruno.config import TEMP_BLOCK_PATH
from naruno.config import TEMP_BLOCKSHASH_PART_PATH
from naruno.config import TEMP_BLOCKSHASH_PATH
from naruno.consensus.finished.transactions.transactions_main import \
    transactions_main
from naruno.consensus.finished.true_time.true_time_main import true_time
from naruno.lib.log import get_logger
from naruno.lib.mix.merkle_root import MerkleTree
from naruno.lib.settings_system import save_settings
from naruno.lib.settings_system import the_settings
from naruno.node.server.server import server
from naruno.transactions.pending_to_validating import PendingtoValidating

logger = get_logger("CONSENSUS")


def finished_main(
    block: Block,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_BLOCKS_PATH: str = None,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,
    custom_server: server = None,
    pass_sync: bool = False,
    dont_clean=False,
) -> None:
    the_server = None
    if custom_server is None:
        the_server = server.Server
    else:
        the_server = custom_server

    the_BLOCKS_PATH = BLOCKS_PATH if custom_BLOCKS_PATH is None else custom_BLOCKS_PATH
    the_TEMP_ACCOUNTS_PATH = (TEMP_ACCOUNTS_PATH
                              if custom_TEMP_ACCOUNTS_PATH is None else
                              custom_TEMP_ACCOUNTS_PATH)
    the_TEMP_BLOCKSHASH_PATH = (TEMP_BLOCKSHASH_PATH
                                if custom_TEMP_BLOCKSHASH_PATH is None else
                                custom_TEMP_BLOCKSHASH_PATH)
    the_TEMP_BLOCKSHASH_PART_PATH = (TEMP_BLOCKSHASH_PART_PATH if
                                     custom_TEMP_BLOCKSHASH_PART_PATH is None
                                     else custom_TEMP_BLOCKSHASH_PART_PATH)

    the_TEMP_BLOCK_PATH = (TEMP_BLOCK_PATH if custom_TEMP_BLOCK_PATH is None
                           else custom_TEMP_BLOCK_PATH)
    block.sync_empty_blocks() if pass_sync is False else None
    if true_time(block):
        logger.debug(
            "Consensus proccess is complated, the block will be reset")

        reset_block = block.reset_the_block()

        settings = the_settings()
        if reset_block != False:
            block2 = reset_block[0]

            new_tx_from_us = False
            new_transactions_list = transactions_main(block2)
            if new_transactions_list is not None:
                SaveBlockstoBlockchainDB(
                    block2,
                    custom_BLOCKS_PATH=the_BLOCKS_PATH,
                    custom_TEMP_ACCOUNTS_PATH=the_TEMP_ACCOUNTS_PATH,
                    custom_TEMP_BLOCKSHASH_PATH=the_TEMP_BLOCKSHASH_PATH,
                    custom_TEMP_BLOCKSHASH_PART_PATH=
                    the_TEMP_BLOCKSHASH_PART_PATH,
                    dont_clean=dont_clean,
                )
                new_tx_from_us = True
                settings["save_blockshash"] = True
                save_settings(settings)
            if block2.sequence_number == 0:
                SaveBlockstoBlockchainDB(
                    block2,
                    custom_BLOCKS_PATH=the_BLOCKS_PATH,
                    custom_TEMP_ACCOUNTS_PATH=the_TEMP_ACCOUNTS_PATH,
                    custom_TEMP_BLOCKSHASH_PATH=the_TEMP_BLOCKSHASH_PATH,
                    custom_TEMP_BLOCKSHASH_PART_PATH=
                    the_TEMP_BLOCKSHASH_PART_PATH,
                    force=True,
                    dont_clean=dont_clean,
                )

            SaveBlockshash(
                reset_block[1].previous_hash,
                custom_TEMP_BLOCKSHASH_PATH=the_TEMP_BLOCKSHASH_PATH,
            )

            the_blocks_hash = GetBlockshash(
                custom_TEMP_BLOCKSHASH_PATH=the_TEMP_BLOCKSHASH_PATH)

            if len(the_blocks_hash) == block.part_amount:
                block.empty_block_number += block.gap_block_number

                block.sync = True
                SaveBlockshash_part(
                    MerkleTree(the_blocks_hash).getRootHash(),
                    custom_TEMP_BLOCKSHASH_PART_PATH=
                    the_TEMP_BLOCKSHASH_PART_PATH,
                )
                the_blockshas_part = GetBlockshash_part(
                    custom_TEMP_BLOCKSHASH_PART_PATH=
                    the_TEMP_BLOCKSHASH_PART_PATH)
                block.part_amount_cache = MerkleTree(
                    the_blockshas_part).getRootHash()
                if settings["save_blockshash"] == True:
                    shutil.copyfile(
                        the_TEMP_BLOCKSHASH_PATH,
                        (the_BLOCKS_PATH + str(block.sequence_number) +
                         ".blockshash_full.json"),
                    )
                    if not new_tx_from_us:
                        settings["save_blockshash"] = False
                        save_settings(settings)
                elif block.sequence_number - 1 == block.part_amount:
                    shutil.copyfile(
                        the_TEMP_BLOCKSHASH_PATH,
                        (the_BLOCKS_PATH + str(block.sequence_number) +
                         ".blockshash_full.json"),
                    )
                os.remove(the_TEMP_BLOCKSHASH_PATH)

                difference = (block.start_time +
                              (block.hard_block_number *
                               block.block_time)) - int(time.time())
                time.sleep(difference)

        PendingtoValidating(block)
        SaveBlock(
            block,
            custom_TEMP_BLOCK_PATH=the_TEMP_BLOCK_PATH,
            custom_TEMP_ACCOUNTS_PATH=the_TEMP_ACCOUNTS_PATH,
            custom_TEMP_BLOCKSHASH_PATH=the_TEMP_BLOCKSHASH_PATH,
            custom_TEMP_BLOCKSHASH_PART_PATH=the_TEMP_BLOCKSHASH_PART_PATH,
        )
        return True
    else:
        if block.sync == True:
            block.sync = False
            SaveBlock(
                block,
                custom_TEMP_BLOCK_PATH=the_TEMP_BLOCK_PATH,
                custom_TEMP_ACCOUNTS_PATH=the_TEMP_ACCOUNTS_PATH,
                custom_TEMP_BLOCKSHASH_PATH=the_TEMP_BLOCKSHASH_PATH,
                custom_TEMP_BLOCKSHASH_PART_PATH=the_TEMP_BLOCKSHASH_PART_PATH,
            )
            [
                the_server.send_block_to_other_nodes(sync_client, sync=True)
                for sync_client in the_server.sync_clients
            ]
            the_server.sync_clients = []

        logger.debug(
            "Consensus proccess is complated, waiting for the true time")
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
from hashlib import sha256

from naruno.wallet.ellipticcurve.math import Math
from naruno.wallet.ellipticcurve.signature import Signature
from naruno.wallet.ellipticcurve.utils.binary import \
    numberFromByteString
from naruno.wallet.ellipticcurve.utils.compatibility import *
from naruno.wallet.ellipticcurve.utils.integer import RandomInteger


class Ecdsa:

    @classmethod
    def sign(cls, message, privateKey, hashfunc=sha256):
        byteMessage = hashfunc(toBytes(message)).digest()
        numberMessage = numberFromByteString(byteMessage)
        curve = privateKey.curve

        r, s, randSignPoint = 0, 0, None
        while r == 0 or s == 0:
            randNum = RandomInteger.between(1, curve.N - 1)
            randSignPoint = Math.multiply(curve.G,
                                          n=randNum,
                                          A=curve.A,
                                          P=curve.P,
                                          N=curve.N)
            r = randSignPoint.x % curve.N
            s = ((numberMessage + r * privateKey.secret) *
                 (Math.inv(randNum, curve.N))) % curve.N
        recoveryId = randSignPoint.y & 1

        return Signature(r=r, s=s, recoveryId=recoveryId)

    @classmethod
    def verify(cls, message, signature, publicKey, hashfunc=sha256):
        byteMessage = hashfunc(toBytes(message)).digest()
        numberMessage = numberFromByteString(byteMessage)
        curve = publicKey.curve
        r = signature.r
        s = signature.s

        inv = Math.inv(s, curve.N)
        u1 = Math.multiply(curve.G,
                           n=(numberMessage * inv) % curve.N,
                           N=curve.N,
                           A=curve.A,
                           P=curve.P)
        u2 = Math.multiply(publicKey.point,
                           n=(r * inv) % curve.N,
                           N=curve.N,
                           A=curve.A,
                           P=curve.P)
        v = Math.add(u1, u2, A=curve.A, P=curve.P)

        return v.x % curve.N == r



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import sqlite3
from urllib.request import urlopen

from naruno.accounts.get_accounts import GetAccounts
from naruno.blockchain.block.get_block import GetBlock
from naruno.lib.settings_system import the_settings
from naruno.wallet.wallet_import import Address


def GetMinimumTransferAmount(block=None, custom_TEMP_BLOCK_PATH=None):
    """
    Returns the minimum transfer amount.
    """
    if the_settings()["baklava"]:
        minimum_transfer_amount = int(
            urlopen(
                "http://test_net.1.naruno.org:8000/blockminumumtransferamount/get/"
            ).read().decode("utf-8"))
    else:
        if block is None:
            try:
                block = GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
            except FileNotFoundError:
                return None
        minimum_transfer_amount = block.minumum_transfer_amount

    return minimum_transfer_amount



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), "..", ".."))

from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from naruno.lib.settings_system import the_settings

logger = get_logger("LIB")


def notification(title, message, raise_plyer=False):
    if the_settings()["mute_notifications"] == True:
        logger.info("Notifications are muted")
        return False

    try:
        if raise_plyer:
            raise ImportError
        else:
            None

        from plyer import notification as plyer_notification
        from plyer.utils import platform

        logger.debug("Notification system is started")
        app_name = "Naruno"
        timeout = 10
        logger.debug(f"app_name: {app_name}")
        logger.debug(f"timeout: {timeout}")
        logger.debug(f"title: {title}")
        logger.debug(f"message: {message}")
        main_folder = get_config()["main_folder"]

        icon = (f"{main_folder}/gui_lib/images/logo_win.ico" if platform
                == "win" else f"{main_folder}/gui_lib/images/logo.png")

        logger.debug(f"icon: {icon}")
        plyer_notification.notify(
            title=title,
            message=message,
            app_icon=icon,
            app_name=app_name,
            timeout=timeout,
        )
    except ImportError:
        logger.debug("Passing notification system (no plyer)")
    except NotImplementedError:
        logger.debug(
            "Passing notification system (no usable implementation found for notification)"
        )


if __name__ == "__main__":
    notification("Bahri Can", "Ergül")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import copy
import threading

from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.finished.finished_main import finished_main
from naruno.consensus.ongoing.ongoing_main import ongoing_main
from naruno.lib.log import get_logger
from naruno.lib.perpetualtimer import perpetualTimer
from naruno.node.client.client import client
from naruno.node.server.server import server
from naruno.transactions.cleaner import Cleaner
from naruno.transactions.pending.get_pending import GetPending

from naruno.consensus.sync.send_block import send_block
from naruno.consensus.sync.send_block_hash import send_block_hash

logger = get_logger("CONSENSUS")



def sync(
    block: Block,
    pending_list_txs: list = None,
    custom_server: server = None,
    send_block_error: bool = False,
    send_block_hash_error: bool = False,
    send_transaction_error: bool = False,
    ):
    """
    Data sending consists of 3 stages. 
    Block sending,blockhash sending and transection.
    It shares the data of the existing chains with the nodes.
    """


    logger.info("Data sending process is starting")
    the_server = server.Server if custom_server is None else custom_server

    if not block.round_1:
        threading.Thread(
            target=send_block,
            args=(
                block, the_server, send_block_error
            ),
        ).start()

    else:
        if not block.round_2:
            threading.Thread(
                target=send_block_hash,
                args=(
                    block, the_server, send_block_hash_error
                ),
            ).start()

    logger.debug("Transactions is sending to the unl nodes")
    the_transactions_list = copy.copy(block.validating_list)
    if pending_list_txs is not None:
        the_transactions_list += pending_list_txs
    for i in the_transactions_list:
        try:
            the_server.send_transaction(i)
            if send_transaction_error:
                raise Exception("Transaction sending error")
        except Exception as e:
            logger.error(f"Transaction sending error: {e}")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import os
import shutil

from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.blocks_hash import GetBlockshash
from naruno.blockchain.block.blocks_hash import GetBlockshash_part
from naruno.blockchain.block.blocks_hash import SaveBlockshash
from naruno.blockchain.block.blocks_hash import SaveBlockshash_part
from naruno.blockchain.block.save_block import SaveBlock
from naruno.config import BLOCKS_PATH
from naruno.config import TEMP_ACCOUNTS_PATH
from naruno.lib.settings_system import the_settings
from naruno.wallet.wallet_import import wallet_import
from naruno.lib.log import get_logger
logger = get_logger("BLOCKCHAIN")

def SaveBlockstoBlockchainDB(
    block,
    custom_BLOCKS_PATH=None,
    custom_TEMP_ACCOUNTS_PATH=None,
    custom_TEMP_BLOCKSHASH_PATH=None,
    custom_TEMP_BLOCKSHASH_PART_PATH=None,
    force=False,
    dont_clean=False,
):
    """
    Adds the block to the blockchain database
    at BLOCKS_PATH.
    """

    logger.info("Saving block to blockchain database...")
    logger.info(f"Block: {block.__dict__}")

    if the_settings()["dont_save_blocks"]:
        logger.debug("Not saving blocks because of settings.")
        return True

    my_public_key = "".join([
        l.strip() for l in wallet_import(-1, 0).splitlines()
        if l and not l.startswith("-----")
    ])
    my_address = wallet_import(-1, 3)
    our_tx = any((validated_transaction.fromUser == my_public_key) or (
        validated_transaction.toUser == my_address)
                 for validated_transaction in block.validating_list)
    if our_tx or force:
        the_BLOCKS_PATH = (BLOCKS_PATH if custom_BLOCKS_PATH is None else
                           custom_BLOCKS_PATH)
        SaveBlock(
            block,
            (the_BLOCKS_PATH + str(block.sequence_number) + ".block.json"),dont_clean=dont_clean)

        the_TEMP_ACCOUNTS_PATH = (TEMP_ACCOUNTS_PATH
                                  if custom_TEMP_ACCOUNTS_PATH is None else
                                  custom_TEMP_ACCOUNTS_PATH)

        shutil.copyfile(
            the_TEMP_ACCOUNTS_PATH,
            (the_BLOCKS_PATH + str(block.sequence_number) + ".accounts.db"),
        )

        shutil.copyfile(
            custom_TEMP_BLOCKSHASH_PATH,
            (the_BLOCKS_PATH + str(block.sequence_number) +
             ".blockshash.json"),
        )

        shutil.copyfile(
            custom_TEMP_BLOCKSHASH_PART_PATH,
            (the_BLOCKS_PATH + str(block.sequence_number) +
             ".blockshashpart.json"),
        )

        return True
    else:
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os

from kivy.core.clipboard import Clipboard
from kivymd.toast import toast
from kivymd.uix.button import MDFlatButton
from kivymd.uix.gridlayout import MDGridLayout
from kivymd.uix.screen import MDScreen
from kivymd_extensions.sweetalert import SweetAlert

import naruno.gui.the_naruno_gui_app
from naruno.blockchain.block.create_block import CreateBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.consensus.consensus_main import consensus_trigger
from naruno.gui.popup import popup
from naruno.lib.perpetualtimer import perpetualTimer
from naruno.lib.qr import qr
from naruno.lib.settings_system import the_settings
from naruno.lib.status import Status
from naruno.node.server.server import server


class NodeScreen(MDScreen):
    pass



class NodeBox(MDGridLayout):
    cols = 2

    def start_node_server_func(self):
        server(
            self.start_node_server_dialog.input_results["IP"],
            int(self.start_node_server_dialog.input_results["PORT"]),
        )

    def show_start_node_server_dialog(self):
        self.start_node_server_dialog = popup(
            title="Start Node Server",
            target=self.start_node_server_func,
            inputs=[
                ["IP", False],
                ["PORT", False],
            ],
        )

    def check_node_server(self):
        if server.Server is None:
            popup(title="Please start the node server", type="failure")
            return False
        return True

    def stop_node_server(self):
        if not self.check_node_server():
            return False
        server.Server.stop()

    def connect_to_know_node(self):
        server.connectionfrommixdb()

    def connect_a_node_func(self):

        if not self.check_node_server():
            return False
        server.Server.connect(
            self.connect_a_node_dialog.input_results["IP"],
            int(self.connect_a_node_dialog.input_results["PORT"]),
        )

    def show_connect_a_node_dialog(self):
        self.connect_a_node_dialog = popup(
            title="Connect a Node",
            target=self.connect_a_node_func,
            inputs=[
                ["IP", False],
                ["PORT", False],
            ],
        )

    def add_unl_node_func(self):

        from naruno.node.unl import Unl

        Unl.save_new_unl_node(
            self.add_unl_node_dialog.input_results["PublicKey"])

    def show_add_unl_node_dialog(self):
        self.add_unl_node_dialog = popup(
            title="Add UNL Node",
            target=self.add_unl_node_func,
            inputs=[
                ["PublicKey", False],
            ],
        )

    def get_block(self):
        if not self.check_node_server():
            return False
        if the_settings()["test_mode"]:
            the_block = CreateBlock()
            SaveBlock(the_block)
            server.Server.send_block_to_other_nodes()
            perpetualTimer(the_block.consensus_timer, consensus_trigger)
        else:
            server.Server.send_me_full_block()

    def nd_id(self):
        Clipboard.copy(server.id)
        popup(title="The ID has been copied to your clipboard.",
              type="success")

    def nd_id_qr(self):
        location_of_qr = qr(server.id)
        popup(text=server.id,
              image=location_of_qr,
              height_image="450px",
              type="qr")

    def status(self):
        toast("Calculating...")
        status = Status()
        if status == "Good":
            popup(title="Good", type="success")

        elif status == "Not bad":
            popup(title="Not bad", type="info")
        elif status == "Bad":
            popup(title="Bad", type="warning")
        elif status == "Very bad":
            popup(title="Very bad", type="warning")
        elif status == "Not work":
            popup(title="Not work", type="failure")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.accounts.get_accounts import GetAccounts
from naruno.blockchain.block.blocks_hash import GetBlockshash
from naruno.blockchain.block.blocks_hash import GetBlockshash_part
from naruno.blockchain.block.get_block import GetBlock
from naruno.config import BLOCKS_PATH


def GetBlockstoBlockchainDB(
    sequence_number,
    custom_BLOCKS_PATH=None,
    custom_TEMP_ACCOUNTS_PATH=None,
    custom_TEMP_BLOCKSHASH_PATH=None,
    custom_TEMP_BLOCKSHASH_PART_PATH=None,
    dont_clean=False,
):
    """
    Gets the block from the blockchain database
    """
    try:

        the_BLOCKS_PATH = (BLOCKS_PATH if custom_BLOCKS_PATH is None else
                           custom_BLOCKS_PATH)

        the_block = GetBlock(
            (the_BLOCKS_PATH + str(sequence_number) + ".block.json"), dont_clean=dont_clean)
        the_accounts = GetAccounts(
            (the_BLOCKS_PATH + str(sequence_number) + ".accounts.db"))
        the_blockshash = GetBlockshash(the_BLOCKS_PATH + str(sequence_number) +
                                       ".blockshash.json")
        the_blockshashpart = GetBlockshash_part(the_BLOCKS_PATH +
                                                str(sequence_number) +
                                                ".blockshashpart.json")
        result = [the_block, the_accounts, the_blockshash, the_blockshashpart]

        return result

    except FileNotFoundError:
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import time

from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import GetBlockshash
from naruno.blockchain.block.blocks_hash import GetBlockshash_part
from naruno.blockchain.block.blocks_hash import SaveBlockshash
from naruno.blockchain.block.blocks_hash import SaveBlockshash_part
from naruno.blockchain.block.hash.calculate_hash import CalculateHash
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.rounds.round_1.checks.checks_main import \
    round_check
from naruno.consensus.rounds.round_1.process.transactions.checks.duplicated import \
    Remove_Duplicates
from naruno.consensus.rounds.round_1.process.transactions.find_newly.find_newly_main import \
    find_newly
from naruno.consensus.rounds.round_1.process.transactions.find_validated.find_validated_main import \
    find_validated
from naruno.lib.log import get_logger
from naruno.node.get_candidate_blocks import GetCandidateBlocks
from naruno.node.server.server import server
from naruno.node.unl import Unl
from naruno.transactions.cleaner import Cleaner
from naruno.transactions.get_transaction import GetTransaction
from naruno.transactions.pending.get_pending import GetPending
from naruno.transactions.process_the_transaction import \
    ProccesstheTransaction

logger = get_logger("CONSENSUS_FIRST_ROUND")


def transactions_main(block: Block, candidate_class: candidate_block,
                      unl_nodes: dict, clean=True) -> list:
    temp_validating_list = find_validated(block,
                                          candidate_class=candidate_class,
                                          unl_nodes=unl_nodes)

    newly_added_list = find_newly(block,
                                  temp_validating_list=temp_validating_list)

    block.validating_list = temp_validating_list

    block = Remove_Duplicates(block)

    pending_list_txs = GetPending()
    if clean:
        cleaned_lists = Cleaner(block, pending_list_txs)
        block.validating_list = cleaned_lists[0]
        pending_list_txs = cleaned_lists[1]

    logger.debug(f"Newly validating list {block.validating_list}")
        
    
    return temp_validating_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
from typing import List

from naruno.config import MY_TRANSACTION_PATH
from naruno.lib.config_system import get_config
from naruno.lib.notification import notification
from naruno.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction
from naruno.transactions.my_transactions.save_my_transaction import \
    SaveMyTransaction
from naruno.transactions.transaction import Transaction
from naruno.wallet.wallet_import import Address


def SavetoMyTransaction(
    tx: Transaction,
    validated: bool = False,
    sended: bool = False,
    custom_currently_list: list = None,
) -> list:
    """
    Saves the transaction to the transaction db.
    Parameters:
        tx: The transaction that is going to be saved.
        validated: The boolean that sets if the transaction is validated or not.
        custom_currently_list: The list for custom situations.
    Returns:
        The list of the my transactions.
    """

    currently_list = (GetMyTransaction() if custom_currently_list is None else
                      custom_currently_list)
    if not tx.signature == "NARUNO":
        new = True

        for tx_list in currently_list:
            if tx_list[0].signature == tx.signature:
                new = False

        if new:
            if not sended and validated:
                notification(
                    "Incoming TX",
                    f"{tx.data}:{tx.amount} from {Address(tx.fromUser)}")
            elif sended and not validated:
                notification("Sended TX",
                             f"{tx.data}:{tx.amount} to {tx.toUser}")
            elif sended and validated:
                notification("Validated TX",
                             f"{tx.data}:{tx.amount} to {tx.toUser}")

            tx_list = [tx, validated, sended]
            currently_list.append(tx_list)

        else:
            for tx_list in currently_list:
                if tx_list[0].signature == tx.signature:
                    tx_list[1] = validated
                    tx_list[2] = sended

    SaveMyTransaction(currently_list)

    return currently_list



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
from hashlib import sha256
from zipfile import ZipFile

from naruno.accounts.get_accounts import GetAccounts
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.get_block_from_blockchain_db import \
    GetBlockstoBlockchainDB
from naruno.config import BLOCKS_PATH
from naruno.config import MY_TRANSACTION_PATH
from naruno.config import PROOF_PATH
from naruno.lib.config_system import get_config
from naruno.transactions.transaction import Transaction


def GetProof(
    signature: str,
    custom_PROOF_PATH=None,
    custom_BLOCKS_PATH=None,
    custom_TEMP_ACCOUNTS_PATH=None,
    custom_TEMP_BLOCKSHASH_PATH=None,
    custom_TEMP_BLOCKSHASH_PART_PATH=None,
) -> str:

    the_PROOF_PATH = PROOF_PATH if custom_PROOF_PATH is None else custom_PROOF_PATH

    the_BLOCKS_PATH = BLOCKS_PATH if custom_BLOCKS_PATH is None else custom_BLOCKS_PATH

    os.chdir(get_config()["main_folder"])
    sequence_number = None

    for file in os.listdir(the_BLOCKS_PATH):
        if file.endswith(".block.json"):
            with open(the_BLOCKS_PATH + file, "r") as block_file:
                the_block_json = json.load(block_file)
            for transaction in the_block_json["validating_list"]:
                if transaction["signature"] == signature:
                    sequence_number = file.split(".")[0]

    if sequence_number is None:
        return None

    result = GetBlockstoBlockchainDB(
        sequence_number,
        custom_BLOCKS_PATH=the_BLOCKS_PATH,
        custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
        custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
        custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
    )
    full_blockshash_sequence_number = result[0].sequence_number + (
        result[0].part_amount - result[0].sequence_number)

    full_blockshash_path = (the_BLOCKS_PATH +
                            str(full_blockshash_sequence_number - 1) +
                            ".blockshash_full.json")

    block_path = the_BLOCKS_PATH + str(sequence_number) + ".block.json"
    account_path = the_BLOCKS_PATH + str(sequence_number) + ".accounts.db"
    blockshash_path = the_BLOCKS_PATH + str(
        sequence_number) + ".blockshash.json"
    blockshashpart_path = (the_BLOCKS_PATH + str(sequence_number) +
                           ".blockshashpart.json")

    proof_path = (the_PROOF_PATH + sha256(
        (signature).encode("utf-8")).hexdigest() + ".proof.zip")

    with ZipFile(proof_path, "w") as zip:
        zip.write(full_blockshash_path)
        zip.write(block_path)
        zip.write(account_path)
        zip.write(blockshash_path)
        zip.write(blockshashpart_path)

    return proof_path



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os

from naruno.config import *
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from naruno.wallet.get_saved_wallet import get_saved_wallet

logger = get_logger("WALLET")


def wallet_delete(account):
    saved_wallet = get_saved_wallet()
    if account in saved_wallet:
        del saved_wallet[account]

        os.chdir(get_config()["main_folder"])
        with open(WALLETS_PATH, "w") as wallet_list_file:
            json.dump(saved_wallet, wallet_list_file, indent=4)
        logger.info(f"Wallet {account} deleted")
        return True
    else:
        logger.error(f"Wallet {account} not found")
        return False



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
import sqlite3

from naruno.accounts.account import Account
from naruno.config import TEMP_ACCOUNTS_PATH
from naruno.lib.config_system import get_config


def GetAccounts(custom_TEMP_ACCOUNTS_PATH=None):
    """
    Returns the accounts from TEMP_ACCOUNTS_PATH.
    """

    the_TEMP_ACCOUNTS_PATH = (TEMP_ACCOUNTS_PATH
                              if custom_TEMP_ACCOUNTS_PATH is None else
                              custom_TEMP_ACCOUNTS_PATH)

    os.chdir(get_config()["main_folder"])
    conn = sqlite3.connect(the_TEMP_ACCOUNTS_PATH, check_same_thread=False)
    c = conn.cursor()
    c.execute(
        """CREATE TABLE IF NOT EXISTS account_list (address text, sequence_number integer, balance integer)"""
    )

    return c



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import csv
import os

from naruno.config import MY_TRANSACTION_EXPORT_PATH
from naruno.lib.config_system import get_config
from naruno.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction


def export_the_transactions(
        custom_transactions: list = None,
        custom_MY_TRANSACTION_EXPORT_PATH: str = None) -> bool:
    """
    Export the transactions to a CSV file.
    """
    obj = GetMyTransaction(
    ) if custom_transactions is None else custom_transactions
    filename = (MY_TRANSACTION_EXPORT_PATH
                if custom_MY_TRANSACTION_EXPORT_PATH is None else
                custom_MY_TRANSACTION_EXPORT_PATH)
    if len(obj) == 0:
        return False
    os.chdir(get_config()["main_folder"])
    with open(filename, "w", newline="", encoding="utf-8") as csvfile:
        fieldnames = list(obj[0][0].__dict__.keys()) + ["validated"
                                                        ] + ["sended"]
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        writer.writeheader()
        for obj in obj:
            trans = obj[0].__dict__
            trans["validated"] = obj[1]
            trans["sended"] = obj[2]
            writer.writerow(trans)
    return True



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import json
import os
import sqlite3

from naruno.config import TEMP_BLOCKSHASH_PART_PATH
from naruno.config import TEMP_BLOCKSHASH_PATH
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger

logger = get_logger("BLOCKCHAIN")


def SaveBlockshash(the_blockshash, custom_TEMP_BLOCKSHASH_PATH=None):
    """
    Saves the blockshash to the TEMP_BLOCKSHASH_PATH.
    """

    os.chdir(get_config()["main_folder"])
    the_TEMP_BLOCKSHASH_PATH = (TEMP_BLOCKSHASH_PATH
                                if custom_TEMP_BLOCKSHASH_PATH is None else
                                custom_TEMP_BLOCKSHASH_PATH)

    conn = sqlite3.connect(the_TEMP_BLOCKSHASH_PATH)
    c = conn.cursor()
    c.execute("""CREATE TABLE IF NOT EXISTS blockshash_list (hash text)""")
    if type(the_blockshash) == list:
        for i in the_blockshash:
            c.execute("""INSERT INTO blockshash_list VALUES (?)""", (i, ))
    else:
        c.execute(
            """INSERT INTO blockshash_list VALUES (?)""",
            (the_blockshash, ),
        )
    conn.commit()
    conn.close()


def SaveBlockshash_part(the_blockshash, custom_TEMP_BLOCKSHASH_PART_PATH=None):
    """
    Saves the blockshash part to the TEMP_BLOCKSHASH_PART_PATH.
    """

    os.chdir(get_config()["main_folder"])
    the_TEMP_BLOCKSHASH_PART_PATH = (TEMP_BLOCKSHASH_PART_PATH if
                                     custom_TEMP_BLOCKSHASH_PART_PATH is None
                                     else custom_TEMP_BLOCKSHASH_PART_PATH)
    conn = sqlite3.connect(the_TEMP_BLOCKSHASH_PART_PATH)
    c = conn.cursor()
    c.execute(
        """CREATE TABLE IF NOT EXISTS blockshash_part_list (hash text)""")
    if type(the_blockshash) == list:
        for i in the_blockshash:
            c.execute("""INSERT INTO blockshash_part_list VALUES (?)""", (i, ))
    else:
        c.execute(
            """INSERT INTO blockshash_part_list VALUES (?)""",
            (the_blockshash, ),
        )
    conn.commit()
    conn.close()


def GetBlockshash(custom_TEMP_BLOCKSHASH_PATH=None):
    """
    Returns the blockshash.
    """
    the_TEMP_BLOCKSHASH_PATH = (TEMP_BLOCKSHASH_PATH
                                if custom_TEMP_BLOCKSHASH_PATH is None else
                                custom_TEMP_BLOCKSHASH_PATH)

    os.chdir(get_config()["main_folder"])

    conn = sqlite3.connect(the_TEMP_BLOCKSHASH_PATH, check_same_thread=False)
    c = conn.cursor()
    c.execute("""CREATE TABLE IF NOT EXISTS blockshash_list (hash text)""")
    c.execute("""SELECT * FROM blockshash_list""")
    result = c.fetchall()

    result = [i[0] for i in result]
    conn.close()

    return result


def GetBlockshash_part(custom_TEMP_BLOCKSHASH_PART_PATH=None):
    """
    Returns the blockshash part.
    """
    the_TEMP_BLOCKSHASH_PART_PATH = (TEMP_BLOCKSHASH_PART_PATH if
                                     custom_TEMP_BLOCKSHASH_PART_PATH is None
                                     else custom_TEMP_BLOCKSHASH_PART_PATH)

    os.chdir(get_config()["main_folder"])

    conn = sqlite3.connect(the_TEMP_BLOCKSHASH_PART_PATH,
                           check_same_thread=False)
    c = conn.cursor()
    c.execute(
        """CREATE TABLE IF NOT EXISTS blockshash_part_list (hash text)""")
    c.execute("""SELECT * FROM blockshash_part_list""")
    result = c.fetchall()

    result = [i[0] for i in result]
    conn.close()

    return result



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import copy
import json
import os
import random
import socket
import time
from hashlib import sha256
from shutil import move
from threading import Thread

from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.change_transaction_fee import ChangeTransactionFee
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.save_block import SaveBlock
from naruno.config import CONNECTED_NODES_PATH
from naruno.config import LOADING_ACCOUNTS_PATH
from naruno.config import LOADING_BLOCK_PATH
from naruno.config import LOADING_BLOCKSHASH_PART_PATH
from naruno.config import LOADING_BLOCKSHASH_PATH
from naruno.config import PENDING_TRANSACTIONS_PATH
from naruno.config import TEMP_ACCOUNTS_PATH
from naruno.config import TEMP_BLOCK_PATH
from naruno.config import TEMP_BLOCKSHASH_PART_PATH
from naruno.config import TEMP_BLOCKSHASH_PATH
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from naruno.lib.mix.merkle_root import MerkleTree
from naruno.node.client.client import client
from naruno.node.unl import Unl
from naruno.transactions.check.check_transaction import CheckTransaction
from naruno.transactions.get_transaction import GetTransaction
from naruno.transactions.transaction import Transaction
from naruno.wallet.ellipticcurve.ecdsa import Ecdsa
from naruno.wallet.ellipticcurve.privateKey import PrivateKey
from naruno.wallet.ellipticcurve.publicKey import PublicKey
from naruno.wallet.ellipticcurve.signature import Signature
from naruno.wallet.wallet_import import wallet_import

logger = get_logger("NODE")

a_block = Block("onur")
buffer_size = 6525 + int(
    (a_block.max_data_size // a_block.max_tx_number) * 1.5)


class server(Thread):
    Server = None
    id = wallet_import(0, 0)

    def __init__(
        self,
        host,
        port,
        save_messages=False,
        test=False,
        custom_variables=False,
        custom_TEMP_BLOCK_PATH=None,
        custom_TEMP_ACCOUNTS_PATH=None,
        custom_TEMP_BLOCKSHASH_PATH=None,
        custom_TEMP_BLOCKSHASH_PART_PATH=None,
        custom_LOADING_BLOCK_PATH=None,
        custom_LOADING_ACCOUNTS_PATH=None,
        custom_LOADING_BLOCKSHASH_PATH=None,
        custom_LOADING_BLOCKSHASH_PART_PATH=None,
        custom_CONNECTED_NODES_PATH=None,
        custom_PENDING_TRANSACTIONS_PATH=None,
        time_control=None,
        custom_id=None,
    ):
        Thread.__init__(self)
        self.running = True

        self.host = host
        self.port = port
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind((self.host, self.port))
        self.sock.listen(1)

        self.clients = []
        self.sync_clients = []

        self.messages = []
        self.our_messages = []
        self.save_messages = save_messages

        self.TEMP_BLOCK_PATH = (TEMP_BLOCK_PATH
                                if custom_TEMP_BLOCK_PATH is None else
                                custom_TEMP_BLOCK_PATH)
        self.TEMP_ACCOUNTS_PATH = (TEMP_ACCOUNTS_PATH
                                   if custom_TEMP_ACCOUNTS_PATH is None else
                                   custom_TEMP_ACCOUNTS_PATH)
        self.TEMP_BLOCKSHASH_PATH = (TEMP_BLOCKSHASH_PATH
                                     if custom_TEMP_BLOCKSHASH_PATH is None
                                     else custom_TEMP_BLOCKSHASH_PATH)
        self.TEMP_BLOCKSHASH_PART_PATH = (
            TEMP_BLOCKSHASH_PART_PATH
            if custom_TEMP_BLOCKSHASH_PART_PATH is None else
            custom_TEMP_BLOCKSHASH_PART_PATH)
        self.LOADING_BLOCK_PATH = (LOADING_BLOCK_PATH
                                   if custom_LOADING_BLOCK_PATH is None else
                                   custom_LOADING_BLOCK_PATH)
        self.LOADING_ACCOUNTS_PATH = (LOADING_ACCOUNTS_PATH
                                      if custom_LOADING_ACCOUNTS_PATH is None
                                      else custom_LOADING_ACCOUNTS_PATH)
        self.LOADING_BLOCKSHASH_PATH = (LOADING_BLOCKSHASH_PATH if
                                        custom_LOADING_BLOCKSHASH_PATH is None
                                        else custom_LOADING_BLOCKSHASH_PATH)
        self.LOADING_BLOCKSHASH_PART_PATH = (
            LOADING_BLOCKSHASH_PART_PATH
            if custom_LOADING_BLOCKSHASH_PART_PATH is None else
            custom_LOADING_BLOCKSHASH_PART_PATH)

        self.CONNECTED_NODES_PATH = (CONNECTED_NODES_PATH
                                     if custom_CONNECTED_NODES_PATH is None
                                     else custom_CONNECTED_NODES_PATH)

        self.PENDING_TRANSACTIONS_PATH = (
            PENDING_TRANSACTIONS_PATH
            if custom_PENDING_TRANSACTIONS_PATH is None else
            custom_PENDING_TRANSACTIONS_PATH)

        self.custom_variables = custom_variables

        self.time_control = 10 if time_control is None else time_control

        self.custom_id = custom_id

        if not test:
            self.__class__.Server = self
            self.start()

    def check_connected(self, host, port):
        logger.info("New node")
        logger.debug(f"self.clients: {self.clients}")
        logger.debug(f"Port: {port}")
        logger.debug(f"Host: {host}")
        for a_client in self.clients:
            if a_client.host == host and a_client.port == port:
                logger.warning("Already connected")
                return True
        logger.info("New connection")
        return False

    def run(self):
        logger.info("Server started and listen new connections")
        self.sock.settimeout(10.0)
        while self.running:
            with contextlib.suppress(socket.timeout):
                conn, addr = self.sock.accept()
                logger.info(
                    f"NODE:{self.host}:{self.port} New connection: {addr}")
                data = conn.recv(1024)
                the_id = server.id if self.custom_id is None else self.custom_id
                conn.send(the_id.encode("utf-8"))
                client_id = data.decode("utf-8")
                if Unl.node_is_unl(client_id):
                    logger.info(f"Added node: {client_id}")
                    self.clients.append(client(conn, addr, client_id, self))
                    self.save_connected_node(addr[0], addr[1], client_id)
                else:
                    logger.info(
                        f"This connection want dont accepted because its not unl: {client_id}"
                    )
            time.sleep(0.01)

    def stop(self):
        self.running = False
        socket.socket(socket.AF_INET, socket.SOCK_STREAM).connect(
            (self.host, self.port))
        for c in self.clients:
            c.stop()
        time.sleep(1)
        for c in self.clients:
            c.join()
        self.sock.close()

    def prepare_message(self, data):
        data["id"] = server.id
        data["timestamp"] = str(time.time())
        sign = Ecdsa.sign(
            str(data),
            PrivateKey.fromPem(wallet_import(0, 1)),
        ).toBase64()

        data["signature"] = sign
        return data

    def send(self, data, except_client=None):
        data = self.prepare_message(data)
        logger.debug(
            f"NODE:{self.host}:{self.port} Send to: {[[a_client.host, a_client.port] for a_client in self.clients]}"
        )
        for a_client in self.clients:
            if a_client != except_client:
                self.send_client(a_client, data, ready_to_send=True)
        try:
            del data["buffer"]
        except KeyError:
            pass
        return data

    def send_client(self, node, data, ready_to_send=False):
        if not ready_to_send:
            data = self.prepare_message(data)
        if len(json.dumps(data).encode("utf-8")) < buffer_size:
            data["buffer"] = "0" * (
                (buffer_size - len(json.dumps(data).encode("utf-8"))) - 14)
        with contextlib.suppress(socket.timeout):
            node.socket.sendall(json.dumps(data).encode("utf-8"))
        with contextlib.suppress(KeyError):
            del data["buffer"]
        time.sleep(0.02)
        if self.save_messages:
            self.our_messages.append(data)
        return data

    def get_message(self, client, data):
        if self.check_message(data):
            logger.debug(f"NODE:{self.host}:{self.port} New message: data")
            if self.save_messages:
                self.messages.append(data)
            self.direct_message(client, data)
        else:
            logger.debug(
                f"NODE:{self.host}:{self.port} Message not valid: data")

    def check_message(self, data):
        if "id" not in data:
            logger.debug("No id")
            logger.debug(data)
            return False
        if "signature" not in data:
            logger.debug("No signature")
            return False
        if "timestamp" not in data:
            logger.debug("No timestamp")
            return False
        # the_control = time.time() - float(data["timestamp"])
        # if the_control > self.time_control:
        #    logger.debug("Time control is not true")
        #    return False
        # remove sign from data
        sign = data["signature"]
        del data["signature"]
        message = str(data)
        data["signature"] = sign

        signature_verify = Ecdsa.verify(
            message,
            Signature.fromBase64(sign),
            PublicKey.fromPem(data["id"]),
        )

        if not signature_verify:
            logger.debug("Signature not valid")
            return False

        return True

    def connect(self, host, port):
        connected = self.check_connected(host=host, port=port)
        if not connected:
            conn = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            conn.settimeout(10.0)
            addr = (host, port)
            conn.connect(addr)
            conn.send(server.id.encode("utf-8"))
            try:
                client_id = conn.recv(1024).decode("utf-8")
                if Unl.node_is_unl(client_id):
                    self.clients.append(client(conn, addr, client_id, self))
                    self.save_connected_node(addr[0], addr[1], client_id)
                    return True
            except socket.timeout:
                logger.info(
                    f"NODE:{self.host}:{self.port} Connection timeout: {addr}")
                conn.close()

    @staticmethod
    def get_connected_nodes(custom_CONNECTED_NODES_PATH=None):
        """
        Returns the connected nodes.
        """

        the_CONNECTED_NODES_PATH = (CONNECTED_NODES_PATH
                                    if custom_CONNECTED_NODES_PATH is None else
                                    custom_CONNECTED_NODES_PATH)

        the_pending_list = {}
        os.chdir(get_config()["main_folder"])
        for entry in os.scandir(the_CONNECTED_NODES_PATH):
            if entry.name != "README.md":
                with open(entry.path, "r") as my_transaction_file:
                    loaded_json = json.load(my_transaction_file)
                    the_pending_list[loaded_json["host"] +
                                     str(loaded_json["port"]) +
                                     loaded_json["id"]] = loaded_json

        return the_pending_list

    def save_connected_node(self, host, port, node_id):
        """
        Saves the connected nodes.
        """

        node_list = {}
        node_list["id"] = node_id
        node_list["host"] = host
        node_list["port"] = port

        node_id = sha256(
            (node_id + host + str(port)).encode("utf-8")).hexdigest()
        file_name = self.CONNECTED_NODES_PATH + f"{node_id}.json"
        os.chdir(get_config()["main_folder"])
        with open(file_name, "w") as connected_node_file:
            json.dump(node_list, connected_node_file, indent=4)

    @staticmethod
    def connectionfrommixdb(custom_server=None,
                            custom_CONNECTED_NODES_PATH=None):
        """
        Connects to the mixdb.
        """
        the_server = server.Server if custom_server is None else custom_server
        the_CONNECTED_NODES_PATH = (the_server.CONNECTED_NODES_PATH
                                    if custom_CONNECTED_NODES_PATH is None else
                                    custom_CONNECTED_NODES_PATH)
        node_list = the_server.get_connected_nodes(
            custom_CONNECTED_NODES_PATH=the_CONNECTED_NODES_PATH)
        for element in node_list:
            with contextlib.suppress(Exception):
                the_server.connect(
                    node_list[element]["host"],
                    node_list[element]["port"],
                )

    def connected_node_delete(self, node):
        """
        Deletes a connected node.
        """
        os.chdir(get_config()["main_folder"])
        node_id = sha256((node["id"] + node["host"] +
                          str(node["port"])).encode("utf-8")).hexdigest()
        for entry in os.scandir(self.CONNECTED_NODES_PATH):
            if entry.name == f"{node_id}.json":
                os.remove(entry.path)

    def direct_message(self, node, data):
        logger.debug("Directing message: {}".format(data["action"]))
        if "sendmefullblock" == data["action"]:
            self.send_block_to_other_nodes(node)

        if "fullblock" == data["action"]:
            self.get_full_chain(data, node)

        if "fullaccounts" == data["action"]:
            self.get_full_accounts(data, node)

        if "fullblockshash" == data["action"]:
            self.get_full_blockshash(data, node)

        if "fullblockshash_part" == data["action"]:
            self.get_full_blockshash_part(data, node)

        if "transactionrequest" == data["action"]:
            self.get_transaction(data, node)

        if "myblock" == data["action"]:
            self.get_candidate_block(data, node)

        if "myblockhash" == data["action"]:
            self.get_candidate_block_hash(data, node)

    def send_me_full_block(self, node=None):
        the_node = node if node is not None else random.choice(self.clients)
        self.send_client(the_node, {"action": "sendmefullblock"})

    def send_my_block(self, block: Block):
        system = block

        new_list = []

        signature_list = []

        for element in system.validating_list:
            tx_json = element.dump_json()
            new_list.append(tx_json)
            signature_list.append(element.signature)

        first_element = [new_list[0]] if len(new_list) > 0 else []

        data = {
            "action": "myblock",
            "transaction": first_element,
            "total_length": len(new_list),
            "sequence_number":
            system.sequence_number + system.empty_block_number,
            "adding": False,
        }

        self.send(data)

        if len(new_list) > 1:
            for element in new_list[1:]:
                data = {
                    "action": "myblock",
                    "transaction": [element],
                    "total_length": len(new_list),
                    "sequence_number":
                    system.sequence_number + system.empty_block_number,
                    "adding": True,
                }

                self.send(data)

    def send_my_block_hash(self, block):
        system = block

        data = {
            "action": "myblockhash",
            "hash": system.hash,
            "previous_hash": system.previous_hash,
            "sequence_number":
            system.sequence_number + system.empty_block_number,
        }

        self.send(data)

    def get_candidate_block(self, data, node: client):
        if node.candidate_block is None:
            node.candidate_block = data
            return
        if data["sequence_number"] > node.candidate_block["sequence_number"]:
            print("if")
            if len(node.candidate_block_history) >= 5:
                node.candidate_block_history.pop(0)

            node.candidate_block_history.append(copy.copy(
                node.candidate_block))
            node.candidate_block = data
        else:
            print("else")
            if node.candidate_block["total_length"] <= data["total_length"]:
                print("else if")
                if node.candidate_block["total_length"] == data[
                        "total_length"]:
                    print("else if if")
                    if data["adding"]:
                        print("else if if if")
                        node.candidate_block["transaction"].append(
                            data["transaction"][0])
                    else:
                        print("else if if else")
                        node.candidate_block = data
                else:
                    print("else if else")
                    node.candidate_block = data

    def get_candidate_block_hash(self, data, node: client):
        if node.candidate_block_hash is None:
            node.candidate_block_hash = data
            return

        data["sender"] = node.id

        if data["sequence_number"] > node.candidate_block_hash[
                "sequence_number"]:
            if len(node.candidate_block_hash_history) >= 5:
                node.candidate_block_hash_history.pop(0)

            node.candidate_block_hash_history.append(
                copy.copy(node.candidate_block_hash))
            node.candidate_block_hash = data
        else:
            if len(node.candidate_block_hash["hash"]) <= len(data["hash"]):
                node.candidate_block_hash = data

    def send_full_chain(self, node=None):
        log_text = ("Sending full chain" if node is None else
                    f"Sending full chain to {node.id}:{node.host}:{node.port}")
        logger.debug(log_text)
        file = open(self.TEMP_BLOCK_PATH, "rb")
        SendData = file.read(1024)
        while SendData:
            data = {
                "action": "fullblock",
                "byte": (SendData.decode(encoding="iso-8859-1")),
            }
            if node is None:
                self.send(data)
            else:
                self.send_client(node, data)

            SendData = file.read(1024)

            if not SendData:
                data = {
                    "action": "fullblock",
                    "byte": "end",
                }
                if node is None:
                    self.send(data)
                else:
                    self.send_client(node, data)

    def send_full_accounts(self, node=None):
        the_TEMP_ACCOUNTS_PATH = self.TEMP_ACCOUNTS_PATH
        file = open(the_TEMP_ACCOUNTS_PATH, "rb")
        SendData = file.read(1024)
        while SendData:
            data = {
                "action": "fullaccounts",
                "byte": (SendData.decode(encoding="iso-8859-1")),
            }

            if node is None:
                self.send(data)
            else:
                self.send_client(node, data)

            SendData = file.read(1024)
            if not SendData:
                data = {"action": "fullaccounts", "byte": "end"}
                if node is None:
                    self.send(data)
                else:
                    self.send_client(node, data)

    def send_full_blockshash(self, node=None):
        the_TEMP_BLOCKSHASH_PATH = self.TEMP_BLOCKSHASH_PATH
        file = open(the_TEMP_BLOCKSHASH_PATH, "rb")
        SendData = file.read(1024)
        while SendData:
            data = {
                "action": "fullblockshash",
                "byte": (SendData.decode(encoding="iso-8859-1")),
            }
            if node is None:
                self.send(data)
            else:
                self.send_client(node, data)

            SendData = file.read(1024)

            if not SendData:
                data = {"action": "fullblockshash", "byte": "end"}
                if node is None:
                    self.send(data)
                else:
                    self.send_client(node, data)

    def send_full_blockshash_part(self, node=None):
        the_TEMP_BLOCKSHASH_PART_PATH = self.TEMP_BLOCKSHASH_PART_PATH
        file = open(the_TEMP_BLOCKSHASH_PART_PATH, "rb")
        SendData = file.read(1024)
        while SendData:
            data = {
                "action": "fullblockshash_part",
                "byte": (SendData.decode(encoding="iso-8859-1")),
            }
            if node is None:
                self.send(data)
            else:
                self.send_client(node, data)

            SendData = file.read(1024)

            if not SendData:
                data = {"action": "fullblockshash_part", "byte": "end"}
                if node is None:
                    self.send(data)
                else:
                    self.send_client(node, data)

    def get_full_chain(self, data, node):
        logger.debug("Getting full chain")
        get_ok = False

        if not os.path.exists(self.TEMP_BLOCK_PATH):
            get_ok = True
        else:
            system = GetBlock(custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH)
            if node.id == system.dowload_true_block:
                get_ok = True

        if get_ok:
            if str(data["byte"]) == "end":
                move(self.LOADING_BLOCK_PATH, self.TEMP_BLOCK_PATH)

                from naruno.consensus.consensus_main import consensus_trigger
                from naruno.lib.perpetualtimer import perpetualTimer

                system = GetBlock(custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH,
                                  get_normal_block=True)

                ChangeTransactionFee(system)

                perpetualTimer(system.consensus_timer, consensus_trigger)
                SaveBlock(
                    system,
                    custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH,
                    custom_TEMP_ACCOUNTS_PATH=self.TEMP_ACCOUNTS_PATH,
                    custom_TEMP_BLOCKSHASH_PATH=self.TEMP_BLOCKSHASH_PATH,
                    custom_TEMP_BLOCKSHASH_PART_PATH=self.
                    TEMP_BLOCKSHASH_PART_PATH,
                )

            else:
                file = open(self.LOADING_BLOCK_PATH, "ab")

                file.write((data["byte"].encode(encoding="iso-8859-1")))
                file.close()

    def get_full_blockshash(self, data, node):
        the_TEMP_BLOCKSHASH_PATH = self.TEMP_BLOCKSHASH_PATH
        get_ok = False

        if not os.path.exists(the_TEMP_BLOCKSHASH_PATH):
            get_ok = True
        else:
            system = GetBlock(custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH)
            if node.id == system.dowload_true_block:
                get_ok = True

        if get_ok:
            if str(data["byte"]) == "end":
                move(self.LOADING_BLOCKSHASH_PATH, the_TEMP_BLOCKSHASH_PATH)
            else:
                file = open(self.LOADING_BLOCKSHASH_PATH, "ab")
                file.write((data["byte"].encode(encoding="iso-8859-1")))
                file.close()

    def get_full_blockshash_part(self, data, node):
        the_TEMP_BLOCKSHASH_PART_PATH = self.TEMP_BLOCKSHASH_PART_PATH
        get_ok = False

        if not os.path.exists(the_TEMP_BLOCKSHASH_PART_PATH):
            get_ok = True
        else:
            system = GetBlock(custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH)
            if node.id == system.dowload_true_block:
                get_ok = True

        if get_ok:
            if str(data["byte"]) == "end":
                move(self.LOADING_BLOCKSHASH_PART_PATH,
                     the_TEMP_BLOCKSHASH_PART_PATH)
            else:
                file = open(self.LOADING_BLOCKSHASH_PART_PATH, "ab")
                file.write((data["byte"].encode(encoding="iso-8859-1")))
                file.close()

    def get_full_accounts(self, data, node):
        the_TEMP_ACCOUNTS_PATH = self.TEMP_ACCOUNTS_PATH
        the_LOADING_ACCOUNTS_PATH = self.LOADING_ACCOUNTS_PATH
        get_ok = False

        if not os.path.exists(the_TEMP_ACCOUNTS_PATH):
            get_ok = True
        else:
            system = GetBlock(custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH)
            if node.id == system.dowload_true_block:
                get_ok = True

        if get_ok:
            if str(data["byte"]) == "end":
                move(the_LOADING_ACCOUNTS_PATH, the_TEMP_ACCOUNTS_PATH)
            else:
                file = open(the_LOADING_ACCOUNTS_PATH, "ab")
                file.write((data["byte"].encode(encoding="iso-8859-1")))
                file.close()

    @staticmethod
    def send_transaction(
        tx,
        custom_current_time=None,
        custom_sequence_number=None,
        custom_balance=None,
        except_client=None,
        custom_server=None,
    ):
        """
        Sends the given transaction to UNL nodes.
        """
        if not tx.signature == "NARUNO":
            data = {
                "action": "transactionrequest",
                "sequence_number": tx.sequence_number,
                "txsignature": tx.signature,
                "fromUser": tx.fromUser,
                "to_user": tx.toUser,
                "data": tx.data,
                "amount": tx.amount,
                "transaction_fee": tx.transaction_fee,
                "transaction_time": tx.transaction_time,
                "custom_current_time": custom_current_time,
                "custom_sequence_number": custom_sequence_number,
                "custom_balance": custom_balance,
            }
            the_server = server.Server if custom_server is None else custom_server
            the_server.send(data, except_client=except_client)

    def get_transaction(self, data, node):
        block = GetBlock(custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH)
        the_transaction = Transaction(
            data["sequence_number"],
            data["txsignature"],
            data["fromUser"],
            data["to_user"],
            data["data"],
            data["amount"],
            data["transaction_fee"],
            data["transaction_time"],
        )
        custom_current_time = None
        custom_sequence_number = None
        custom_balance = None
        if self.custom_variables:
            custom_current_time = data["custom_current_time"]
            custom_sequence_number = data["custom_sequence_number"]
            custom_balance = data["custom_balance"]
        logger.debug(
            f"NODE:{self.host}:{self.port} -{custom_current_time}-{custom_sequence_number}-{custom_balance}"
        )
        if GetTransaction(
                block,
                the_transaction,
                custom_current_time=custom_current_time,
                custom_sequence_number=custom_sequence_number,
                custom_balance=custom_balance,
                custom_PENDING_TRANSACTIONS_PATH=self.
                PENDING_TRANSACTIONS_PATH,
        ):
            logger.debug(f"NODE:{self.host}:{self.port} Transaction accepted")

            server.send_transaction(the_transaction,
                                    except_client=node,
                                    custom_server=self)
            SaveBlock(
                block,
                custom_TEMP_BLOCK_PATH=self.TEMP_BLOCK_PATH,
                custom_TEMP_ACCOUNTS_PATH=self.TEMP_ACCOUNTS_PATH,
                custom_TEMP_BLOCKSHASH_PATH=self.TEMP_BLOCKSHASH_PATH,
                custom_TEMP_BLOCKSHASH_PART_PATH=self.
                TEMP_BLOCKSHASH_PART_PATH,
            )

    def send_block_to_other_nodes(self, node=None, sync=False):
        """
        Sends the block to the other nodes.
        """
        if node is None or sync:
            self.send_full_accounts(node=node)
            self.send_full_blockshash(node=node)
            self.send_full_blockshash_part(node=node)
            self.send_full_chain(node=node)
        else:
            self.sync_clients.append(node)

    def get_ip(self):
        """
        Returns the IP address of the socket in this class.
        """
        ip = self.sock.getsockname()[0]
        return ip



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import copy
import json
import os
import time
from hashlib import sha256

from naruno.config import MY_TRANSACTION_PATH
from naruno.lib.config_system import get_config
from naruno.transactions.my_transactions.get_my_transaction import \
    GetMyTransaction


def SaveMyTransaction(transaction_list, clear=False):
    """
    Saves the transaction_list to the transaction db.
    """

    os.chdir(get_config()["main_folder"])

    if type(transaction_list) is list:
        entry_name_list = []
        for tx in transaction_list:
            name = copy.copy(tx[0].signature.encode("utf-8"))
            if tx[0].signature == b"":
                name = "empty".encode("utf-8")

            entry_name_list.append(
                os.path.join(MY_TRANSACTION_PATH,
                             sha256(name).hexdigest()))

        new_dict = {
            tx[0].signature: {
                "tx": tx[0].dump_json(),
                "validated": tx[1],
                "sended": tx[2],
            }
            for tx in transaction_list
        }

        if clear:
            for entry in os.scandir(MY_TRANSACTION_PATH):
                if (entry.name != "README.md"
                        and not entry.name.startswith("validated")
                        and not entry.name.startswith("sended")):
                    if entry.name not in str(entry_name_list):
                        os.remove(entry.path)
                        if os.path.exists(
                                os.path.join(MY_TRANSACTION_PATH,
                                             "validated" + entry.name)):
                            os.remove(
                                os.path.join(MY_TRANSACTION_PATH,
                                             "validated" + entry.name))
                        if os.path.exists(
                                os.path.join(MY_TRANSACTION_PATH,
                                             "sended" + entry.name)):
                            os.remove(
                                os.path.join(MY_TRANSACTION_PATH,
                                             "sended" + entry.name))

        transaction_list = new_dict

        for tx in transaction_list:
            name = copy.copy(tx.encode("utf-8"))
            if tx == b"":
                name = "empty".encode("utf-8")

            with open(
                    os.path.join(MY_TRANSACTION_PATH,
                                 sha256(name).hexdigest()),
                    "w") as my_transaction_file:
                json.dump(transaction_list[tx]["tx"], my_transaction_file)
            if transaction_list[tx]["validated"]:
                with open(
                        os.path.join(MY_TRANSACTION_PATH,
                                     "validated" + sha256(name).hexdigest()),
                        "w",
                ) as my_transaction_file:
                    my_transaction_file.write("1")

            if transaction_list[tx]["sended"]:
                with open(
                        os.path.join(MY_TRANSACTION_PATH,
                                     "sended" + sha256(name).hexdigest()),
                        "w",
                ) as my_transaction_file:
                    my_transaction_file.write("1")
    elif type(transaction_list) is dict and transaction_list != {}:
        with open(
                os.path.join(MY_TRANSACTION_PATH,
                             sha256(
                                 transaction_list[0].signature).hexdigest()),
                "w",
        ) as my_transaction_file:
            json.dump(transaction_list[0].dump_json(), my_transaction_file)

        if transaction_list[1]:
            with open(
                    os.path.join(
                        MY_TRANSACTION_PATH,
                        "validated" +
                        sha256(transaction_list[0].signature).hexdigest(),
                    ),
                    "w",
            ) as my_transaction_file:
                my_transaction_file.write("1")
        if transaction_list[2]:
            with open(
                    os.path.join(
                        MY_TRANSACTION_PATH,
                        "sended" +
                        sha256(transaction_list[0].signature).hexdigest(),
                    ),
                    "w",
            ) as my_transaction_file:
                my_transaction_file.write("1")



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import os
import time

from naruno.accounts.get_accounts import GetAccounts
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import GetBlockshash
from naruno.blockchain.block.blocks_hash import GetBlockshash_part
from naruno.blockchain.block.blocks_hash import SaveBlockshash
from naruno.blockchain.block.blocks_hash import SaveBlockshash_part
from naruno.blockchain.block.hash.calculate_hash import CalculateHash
from naruno.blockchain.block.save_block import SaveBlock
from naruno.blockchain.candidate_block.candidate_block_main import \
    candidate_block
from naruno.consensus.rounds.round_1.checks.checks_main import \
    round_check
from naruno.consensus.rounds.round_1.process.transactions.checks.duplicated import Remove_Duplicates
from naruno.consensus.rounds.round_1.process.transactions.transactions_main import \
    transactions_main
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from naruno.node.get_candidate_blocks import GetCandidateBlocks
from naruno.node.server.server import server
from naruno.node.unl import Unl
from naruno.transactions.get_transaction import GetTransaction
from naruno.transactions.process_the_transaction import \
    ProccesstheTransaction
from naruno.config import TEMP_BLOCK_PATH

logger = get_logger("CONSENSUS_FIRST_ROUND")


def round_process(
    block: Block,
    candidate_class: candidate_block,
    unl_nodes: dict,
    custom_TEMP_BLOCK_PATH: str = None,
    custom_TEMP_ACCOUNTS_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PATH: str = None,
    custom_TEMP_BLOCKSHASH_PART_PATH: str = None,
    custom_shares=None,
    custom_fee_address=None,
    clean=True
) -> Block:
    logger.info("Processing for round 1 is started")
    logger.debug(f"First block: {block.dump_json()}")
    transactions_main(block,
                      candidate_class=candidate_class,
                      unl_nodes=unl_nodes, clean=clean)

    block.round_1 = True
    block.round_2_starting_time = block.start_time + block.round_1_time

    account_list = GetAccounts(
        custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH)
    block = ProccesstheTransaction(
        block,
        account_list,
        custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
        custom_shares=custom_shares,
        custom_fee_address=custom_fee_address,
    )

    part_of_blocks_hash = GetBlockshash_part(
        custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH)
    the_blocks_hash = GetBlockshash(
        custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH)
    logger.debug(f"part_of_blocks_hash: {part_of_blocks_hash}")
    logger.debug(f"the_blocks_hash: {the_blocks_hash}")
    logger.debug(f"account_list: {account_list}")
    block.hash = CalculateHash(block, part_of_blocks_hash, the_blocks_hash,
                               account_list)

    logger.debug(f"Block hash {block.hash}")



    


    SaveBlock(
        block,
        custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH,
        custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
        custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
        custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        delete_old_validating_list=True,
    )
    logger.debug(f"End block: {block.dump_json()}")
    return block



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import sqlite3
from urllib.request import urlopen

from naruno.accounts.get_accounts import GetAccounts
from naruno.blockchain.block.get_block import GetBlock
from naruno.blockchain.block.get_minumum_transfer_amount import \
    GetMinimumTransferAmount
from naruno.lib.settings_system import the_settings
from naruno.transactions.pending.get_pending import GetPending
from naruno.wallet.wallet_import import Address


def GetBalance(user,
               account_list=None,
               dont_convert=False,
               block=None,
               custom_TEMP_BLOCK_PATH=None):
    """
    Returns the users balance.
    """
    address = Address(user) if not dont_convert else user

    balance = GetMinimumTransferAmount(
        block=block, custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)

    if the_settings()["baklava"]:
        balance = float(
            urlopen(
                f"http://test_net.1.naruno.org:8000/balance/get/?address={address}"
            ).read().decode("utf-8").replace("\n", ""))
    else:
        if block is None:
            try:
                block = GetBlock(custom_TEMP_BLOCK_PATH=custom_TEMP_BLOCK_PATH)
            except FileNotFoundError:
                return None

        balance = -block.minumum_transfer_amount

        the_account_list = GetAccounts(
        ) if account_list is None else account_list
        the_account_list.execute(
            f"SELECT * FROM account_list WHERE address = '{address}'")
        for row in the_account_list.fetchall():
            balance += row[2]
            break
        if not block.just_one_tx:
            for tx in block.validating_list + GetPending():
                if Address(tx.fromUser) == user:
                    balance -= float(tx.amount) + float(tx.transaction_fee)
                elif tx.toUser == user:
                    balance += float(tx.amount)
    return balance



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
import contextlib
import json
import os
import time

from naruno.accounts.account import Account
from naruno.accounts.save_accounts import SaveAccounts
from naruno.blockchain.block.block_main import Block
from naruno.blockchain.block.blocks_hash import SaveBlockshash
from naruno.blockchain.block.blocks_hash import SaveBlockshash_part
from naruno.config import TEMP_BLOCK_PATH
from naruno.consensus.rounds.round_1.process.transactions.checks.duplicated import \
    Remove_Duplicates
from naruno.lib.config_system import get_config
from naruno.lib.log import get_logger
from naruno.lib.settings_system import the_settings
from naruno.transactions.cleaner import Cleaner
from naruno.transactions.pending.get_pending import GetPending

logger = get_logger("BLOCKCHAIN")


def SaveBlock(
    block: Block,
    custom_TEMP_BLOCK_PATH=None,
    custom_TEMP_ACCOUNTS_PATH=None,
    custom_TEMP_BLOCKSHASH_PATH=None,
    custom_TEMP_BLOCKSHASH_PART_PATH=None,
    delete_old_validating_list=False,
    just_save_normal=False,
    dont_clean=False,
):
    """
    Saves the current block to the TEMP_BLOCK_PATH.
    """
    if not dont_clean:
        if not block.round_1 and not block.round_2:
            cleaned = Cleaner(block, pending_list_txs=GetPending())
            block.validating_list = cleaned[0]

            block = Remove_Duplicates(block)
            block.validating_list = sorted(block.validating_list,
                                       key=lambda x: x.fromUser)

    logger.info("Saving block to disk")
    logger.debug(
        f"Block#{block.sequence_number + block.empty_block_number}:{block.empty_block_number}: {block.dump_json()}"
    )
    if block.first_time:
        accounts_list = [Account(block.creator, block.coin_amount)]
        baklava_test_net_users = [
            Account("55de207a538855b4da2d60325e8afadc3b3caa04",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("86e9a2454e2d2bd12f56ef37e39d026608013e72",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),


            Account("2f58be5d152490affa05a7b0fd3cef8c195dae6d",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("a26536e07f3c2a850fb2b63cbe99d84589674634",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("0be5c9cd8bf68cafeec3a2d5d51678923780d3ff",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("82d87a6bfd279d30ad4894912eae2efacd4d46d6",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("f6e4955a8077ae5ed7d95014b41f22dcee6c0d76",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("73672aafc1890fc18d9b88105380b396eca799a5",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("83a15e056f98305418ee9ea26caf664c3d020040",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),       
            Account("b1df8deda30d4f88cb905ecd57ed0fc7f2021d00",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),  
            Account("ec29c2e01987796a3677da2e3a9b4a098b93b89a",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),  
            Account("887af3d44bfe39005b4cc480c2b03a11c2fb8b63",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),  
            Account("17d3d3e20bd84ddf6e3ed85fa693c12654f174eb",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("1da75d769ab3604abc04763d20dc3f70bf1c69b8",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),    

            Account("d7eee170a14b99e37a3e3fc6d375d1d28bddf62b",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),  
            Account("d7f20b7990cc593e248f1d0dd31dd7a235a40d9a",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),  
            Account("75f7e0e090834c959f6538b1a4c80f03be410bdd",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),                                                              


            Account("d37cb2c0df30965f2bc12cea040386e70e32402b",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   

            Account("0af54b3f47fc1577688e7c2c227672f610cad292",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),       






            Account("96bb7bac1af450ea0c17300ea4f61b1cd0b88b6d",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   
            Account("709bf017a48d2f02bb5d5d8d205ccf39d8205b4a",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   
            Account("5b943d77a8b7e66aa60b98a2197f2197db4f464b",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   
            Account("bba1e2f5871c02b130416229c02dd54fc404cd21",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   
            Account("dcf715a42784bfedf009b515ada46c1946a3339b",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   
            Account("0af54b3f47fc1577688e7c2c227672f610cad292",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   
            Account("0af54b3f47fc1577688e7c2c227672f610cad292",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),   
            Account("0af54b3f47fc1577688e7c2c227672f610cad292",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),                                                                                                                                               

            Account("cbdeeab5577f6f8693e571494e61dc0f356d9d09",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),


            Account("09375da881e7d546b8520a7e39160fc05eb60ce1",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("b3232a5ed1d57c339fb3a258be40c51e221b48af",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("7be509d428fc848485683a599cc4bbd958bd6df3",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("2b76e7031353e9cbf62e2d259b54b98673f997c9",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("614deef9abde68a0dab39fe5ee9f9cb2f9e8773a",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("44281e3b85497d41d8c99f3b8b66a3e88e354ca1",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("5db89649f397be7d3787cfca2e9f39245ef5f8e1",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("b3e6e03b6c3f6ea8989177d23a9a055e2a05659b",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("bc512bab30167fe84c84208be8759bf9839d7815",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("138c27b39d5789319aa9a0c00262cb5360d146d0",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("0dbe3d640ee1632b222154197a23319870f3d12d",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            Account("4a1b3198530952ee851722b105770ef21c59e472",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),



            Account("541cb5cc142e94aed187f83ee0d91a1470db3023",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("caefad8f89b701d8087945780ad24fe0f993ae2f",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("8f571575a492ef27f0fe02916303ea867f50876d",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("508c9565bac49d318eb5c67f6e6e0354acbc4edf",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),

            
            Account("c7b71bd5100d2081de69497776064589e98c4fe1",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("c58b9d12474108867f4a0924f4c521119a57d62f",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("0d435e7fb86a46f4849a4eb41f683ab322f03d6b",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("2E6A70110890b7A46F7857E2866023043CeF4680",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
                                                                  
            Account("078e6dd15b33411462d284baf4c901cfd828558c",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("88b73c67bf3aab4764d627cc9aa1150c6ae97794",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
                                                                  
                                                                                                   
            
            Account("450305d3bb1a0fd7343824f136177220d436c046",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
            Account("e7b72ca5af68bc5b8ddd98e4187c68efcfbec5cd",
                    (2 * block.minumum_transfer_amount) + block.transaction_fee * 100),
                                                           


        ]
        if the_settings()["baklava_users"]:
            accounts_list.extend(baklava_test_net_users)
        SaveAccounts(
            accounts_list,
            custom_TEMP_ACCOUNTS_PATH=custom_TEMP_ACCOUNTS_PATH,
        )
        SaveBlockshash(
            block.previous_hash,
            custom_TEMP_BLOCKSHASH_PATH=custom_TEMP_BLOCKSHASH_PATH,
        )
        SaveBlockshash_part(
            [block.previous_hash],
            custom_TEMP_BLOCKSHASH_PART_PATH=custom_TEMP_BLOCKSHASH_PART_PATH,
        )
        block.first_time = False
    the_TEMP_BLOCK_PATH = (TEMP_BLOCK_PATH if custom_TEMP_BLOCK_PATH is None
                           else custom_TEMP_BLOCK_PATH)
    secondly_situation = 0
    if block.round_1:
        secondly_situation += 1
    if block.round_2:
        secondly_situation += 1
    highest_the_TEMP_BLOCK_PATH = (the_TEMP_BLOCK_PATH + "-" +
                                   str(block.sequence_number + block.empty_block_number) + "-" +
                                   str(len(block.validating_list)) + "-" +
                                   str(secondly_situation) + "-" +
                                   str(time.time()))
    logger.info(f"Saving block to {highest_the_TEMP_BLOCK_PATH}")

    if delete_old_validating_list:
        os.chdir(get_config()["main_folder"])
        for file in os.listdir("db/"):
            if ("db/" + file).startswith(the_TEMP_BLOCK_PATH) and not (
                    "db/" + file) == the_TEMP_BLOCK_PATH:
                number = int((("db/" + file).replace(the_TEMP_BLOCK_PATH,
                                                     "")).split("-")[1])
                high_number = int(
                    (("db/" + file).replace(the_TEMP_BLOCK_PATH,
                                            "")).split("-")[2])
                secondly_situation_number = int(
                    (("db/" + file).replace(the_TEMP_BLOCK_PATH,
                                            "")).split("-")[3])
                if (number == block.sequence_number + block.empty_block_number
                        and high_number != len(block.validating_list)
                        and secondly_situation_number == 1):
                    with contextlib.suppress(FileNotFoundError):
                        logger.info(f"Deleting old validating list: {file}")
                        os.remove("db/" + file)

    for file in os.listdir("db/"):
        if ("db/" + file).startswith(the_TEMP_BLOCK_PATH) and not (
                "db/" + file) == the_TEMP_BLOCK_PATH:
            number = int((("db/" + file).replace(the_TEMP_BLOCK_PATH,
                                                 "")).split("-")[1])  # seq
            high_number = int(
                (("db/" + file).replace(the_TEMP_BLOCK_PATH,
                                        "")).split("-")[2])  # val
            if number < block.sequence_number + block.empty_block_number:
                with contextlib.suppress(FileNotFoundError):
                    logger.info("Removing " + "db/" + file)
                    os.remove("db/" + file)

    with open(the_TEMP_BLOCK_PATH, "w") as block_file:
        json.dump(block.dump_json(), block_file)
    if not just_save_normal:
        with open(highest_the_TEMP_BLOCK_PATH, "w") as block_file:
            json.dump(block.dump_json(), block_file)



#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from naruno.lib.log import get_logger
from naruno.wallet.ellipticcurve.ecdsa import Ecdsa
from naruno.wallet.ellipticcurve.publicKey import PublicKey
from naruno.wallet.ellipticcurve.signature import Signature

logger = get_logger("TRANSACTIONS")


def Check_Sign(transaction):
    """
    Check if the transaction signature is valid
    """

    if Ecdsa.verify(
        (str(transaction.sequence_number) + str(transaction.fromUser) +
         str(transaction.toUser) + str(transaction.data) +
         str(transaction.amount) + str(transaction.transaction_fee) +
         str(transaction.transaction_time)),
            Signature.fromBase64(transaction.signature),
            PublicKey.fromPem(transaction.fromUser),
    ):
        pass
    else:
        logger.debug("The signature is not valid")
        return False

    return True
